; File "XCON8 ROM.ASM"
;============================================================================
;  This is a re-creation of the original Heathkit H8 XCON/8 EPROM code by:
;
;	J. G. Letwin, 05/01/76
;	For *WINTEK* Inc.
;
;	Copyright 05/1976,	WINTEK CORPORATION
;				902 N. 9TH ST.
;				LAFAYETTE, IND.
;
; As modified:	25 Aug 79	PAM8GO	JWTittsier
;					added single button boot from H17
;		31 Dec 79	PAM8AT	JWTittsier
;					added automatic power-on boot from H17
;		04 Mar 80	PAM8GO	JWTittsier
;					changed default display to PC
;		20 Jun 80	RAM8GO	JWTittsier
;					added RAM at zero capability
;		07 Aug 80	Ram8Go	G. Chandler
;					Issue:	01.02.00
;					H17 Boot
;					H47 Boot
;					Auto-Boot
;					Primary/Secondary Support
;					Modified RAM at Zero (no double move)
;					Remove 030.000 default PC to avoid confusion
;
;  At some point, the 07AUG80 issued code was merged with the H17 ROM program, and the
; resulting 4K EPROM was referred to as XCON/8, with a Heath part number of 444-70.
; This is the starting point for the current re-creation.
;
;  Source code recreated:  Sep 2009	T. Gulczynski
;
;  The basic ROM program for the Heath H8 computer contains three elements
; essential to the H8: the PAM-8 monitor, PAM-8 extensions, and the H17 Floppy
; disk code.  The XCON8 EPROM contains all three elements.
;
;  The project to re-create the XCON8 source code was prompted by two major
; factors: Les Bird's H8-Z80 project, and the desire to incorporate an IDE
; interface in the H8.
;
;  An IDE interface, for maximum versatility, will require modifications to
; ROM code for system boot purposes.  The H8/Z80-based CPU board can run the
; original XCON8 code, but a GIDE interface is incorporated in the CPU board
; design, so EPROM support is needed for that, too.
;
;  The XCON8 binary code has long been available in PDF document format, and
; is easily copied from one EPROM to another, but it has never before been
; available in source code format.  Obviously, the source code is needed as
; the starting point for the IDE boot driver.
;
;  There is a limited amount of space in the XCON8 EPROM, and none of it is
; free for use by the GIDE boot subsystem.  Something has to give, and it
; was decided to swap out the H47 boot code in favor of the GIDE code.
; Rather than 'create' a 'new' ROM code baseline that includes GIDE boot code,
; we elected to simply build in a compile-time equate that would allow for H47
; or GIDE boot code, based on the TRUE or FALSE status of the equate.  In this
; manner, the original XCON8 ROM code can be compiled, or a newer version
; with GIDE boot code can be compiled.

false		equ	0
true		equ	! false

GideBoot	equ	false	; Set TRUE for GIDE boot, FALSE for H47 boot

; These equates are needed to generate the offseet addresses used by
; the H17 ROM after being relocated from the compile location of 800H to
; the execution location of 1800H.

I$16FF  EQU     16FFH   ; ----I
I$1800  EQU     1800H   ; ----I
J$181A  EQU     181AH   ; J----
J$181E  EQU     181EH   ; J----
J$1826  EQU     1826H   ; J----

C$18AA  EQU     18AAH   ; -C---
J$18BA  EQU     18BAH   ; J----


C_192C  EQU     192CH   ; -C---
J$194B  EQU     194BH   ; J----
J$195B  EQU     195BH   ; J----
J$1973  EQU     1973H   ; J----
J$198B  EQU     198BH   ; J----
C_19BD  EQU     19BDH   ; JC---
J$19C5  EQU     19C5H   ; J----
I$19D0  EQU     19D0H   ; ----I
C$19D8  EQU     19D8H   ; -C---
J$19FF  EQU     19FFH   ; J----
J$1A1D  EQU     1A1DH   ; J----
J$1A2D  EQU     1A2DH   ; J----
J$1A39  EQU     1A39H   ; J----
J$1A65  EQU     1A65H   ; J----
J$1A71  EQU     1A71H   ; J----
J$1A75  EQU     1A75H   ; J----
C_1A85  EQU     1A85H   ; -C---
J$1AAA  EQU     1AAAH   ; J----
J$1AB9  EQU     1AB9H   ; J----
J$1AD3  EQU     1AD3H   ; J----
C$1AF1  EQU     1AF1H   ; -C---
C_1AF2  EQU     1AF2H   ; -C---
J$1B3B  EQU     1B3BH   ; J----
J$1B71  EQU     1B71H   ; J----
C$1B7D  EQU     1B7DH   ; -C---
J$1B7F  EQU     1B7FH   ; J----
J$1B8B  EQU     1B8BH   ; J----
C$1B9B  EQU     1B9BH   ; -C---

J$1BFE  EQU     1BFEH   ; J----
J_1C38  EQU     1C38H   ; J----
C_1C3F  EQU     1C3FH   ; JC---
J$1C4A  EQU     1C4AH   ; J----
J$1C52  EQU     1C52H   ; J----
J_1C56  EQU     1C56H   ; J----
J$1C58  EQU     1C58H   ; J----
J$1C70  EQU     1C70H   ; J----
J$1C77  EQU     1C77H   ; J----
J$1C85  EQU     1C85H   ; J----
J$1C8C  EQU     1C8CH   ; J----
J$1CB1  EQU     1CB1H   ; J----
J$1CBA  EQU     1CBAH   ; J----
J_1CC0  EQU     1CC0H   ; J----
J_1CCD  EQU     1CCDH   ; J----

J_1CFD  EQU     1CFDH   ; J----
J$1CFF  EQU     1CFFH   ; J----
J$1D0E  EQU     1D0EH   ; J----
J$1D1C  EQU     1D1CH   ; J----
J$1D52  EQU     1D52H   ; J----
J$1D5A  EQU     1D5AH   ; J----

J$1D7F  EQU     1D7FH   ; J----
J$1D91  EQU     1D91H   ; J----

J$1DC8  EQU     1DC8H   ; J----
J$1DCE  EQU     1DCEH   ; J----

J$1DDC  EQU     1DDCH   ; J----
J_1E07  EQU     1E07H   ; J----
J$1E0C  EQU     1E0CH   ; J----
J$1E15  EQU     1E15H   ; J----
J$1E1A  EQU     1E1AH   ; J----
J$1E1F  EQU     1E1FH   ; J----

J$1E58  EQU     1E58H   ; J----
J$1E65  EQU     1E65H   ; J----

J$1E81  EQU     1E81H   ; J----
J_1E92  EQU     1E92H   ; J----
J$1E9B  EQU     1E9BH   ; J----
J$1E9D  EQU     1E9DH   ; J----

J$1ED1  EQU     1ED1H   ; J----
J$1EDE  EQU     1EDEH   ; J----

C_1EED  EQU     1EEDH   ; JC---

J$1EFC  EQU     1EFCH   ; J----

J$1F30  EQU     1F30H   ; J----

C$1FC6  EQU     1FC6H   ; -C---

; Data storage locations for XCON/8 and H17 ROMs.

I$200A  EQU     200AH   ; ----I
I_2010  EQU     2010H   ; ----I

D_2020  EQU     2020H   ; --S--
D_203E  EQU     203EH   ; --SL-
I$2040  EQU     2040H   ; ----I
D$204A  EQU     204AH   ; ---L-
D$204B  EQU     204BH   ; ---L-
D$204C  EQU     204CH   ; ---L-
D$204D  EQU     204DH   ; ---L-
D$204E  EQU     204EH   ; ---L-
D$204F  EQU     204FH   ; ---L-
D$2050  EQU     2050H   ; ---L-
D$2051  EQU     2051H   ; ---L-
D$2052  EQU     2052H   ; ---L-
D$2053  EQU     2053H   ; ---L-
D$2054  EQU     2054H   ; ---L-
D_2055  EQU     2055H   ; --SL-
;				SYDD  EQU     2058H   ; -C---
J$205B  EQU     205BH   ; J----
J_205E  EQU     205EH   ; J----
J_2064  EQU     2064H   ; J----
J$206A  EQU     206AH   ; J----
J$206D  EQU     206DH   ; J----
C_2070  EQU     2070H   ; -C---
C_2073  EQU     2073H   ; -C---
C_2076  EQU     2076H   ; -C---
C$2079  EQU     2079H   ; -C---
C_207C  EQU     207CH   ; -C---
C_207F  EQU     207FH   ; -C---
C_2082  EQU     2082H   ; -C---
C_2085  EQU     2085H   ; -C---
D$2086  EQU     2086H   ; --S--
C_2088  EQU     2088H   ; JC---
C_208B  EQU     208BH   ; -C---
C_208E  EQU     208EH   ; JC---
C_2091  EQU     2091H   ; -C---
C$2094  EQU     2094H   ; -C---
C_2097  EQU     2097H   ; JC---
C_209A  EQU     209AH   ; -C---
C$209D  EQU     209DH   ; -C---
D_20A1  EQU     20A1H   ; --S-I
D_20A4  EQU     20A4H   ; ---LI
D_20A5  EQU     20A5H   ; --SL-
D_20A7  EQU     20A7H   ; --SL-
I$20A9  EQU     20A9H   ; ----I
D_20B2  EQU     20B2H   ; --SL-
I$20B5  EQU     20B5H   ; ----I
I$20B7  EQU     20B7H   ; ----I
D_20BB  EQU     20BBH   ; --SL-
D_20BD  EQU     20BDH   ; --SL-
D$20D0  EQU     20D0H   ; ---L-
D$20D2  EQU     20D2H   ; ---L-
D_20F0  EQU     20F0H   ; ---L-
D$20F1  EQU     20F1H   ; --S--
D$20F2  EQU     20F2H   ; ---L-
D_20F4  EQU     20F4H   ; --SL-
D$20F6  EQU     20F6H   ; ---L-
D$20F8  EQU     20F8H   ; ---L-
I_20F9  EQU     20F9H   ; ----I
D_20FA  EQU     20FAH   ; --SL-
D_20FC  EQU     20FCH   ; --SL-
D_20FE  EQU     20FEH   ; ---L-
X_2100  EQU     2100H   ; J-S--
D_2102  EQU     2102H   ; ---L-
D$2104  EQU     2104H   ; ---L-
C_210B  EQU     210BH   ; -C---
D_2124  EQU     2124H   ; ---L-
D$2125  EQU     2125H   ; ---L-
D_2126  EQU     2126H   ; ---L-
D_2127  EQU     2127H   ; --SL-
D_2128  EQU     2128H   ; ---LI
D_2129  EQU     2129H   ; ---LI
D$213F  EQU     213FH   ; ---L-
D_2149  EQU     2149H   ; --SLI
D$214B  EQU     214BH   ; --S--
D_214C  EQU     214CH   ; --SL-

; H17 Constants

; H17 ROM routine addresses
R_SDP_  equ     1E3BH

; Addresses in Resident HDOS code
SYDD	equ	2058h

S_FASER	equ	210BH




; Data bytes in low RAM
Start	equ	2000h
IoWrk	equ	2002h		; I/O Work Area

XinB	equ	2004h
PrsRAM	equ	2004h
RegI	equ	2005h
DspRot	equ	2006h
DspMod	equ	2007h
MFlag	equ	2008h
CtlFlg	equ	2009h

ALeds	equ	200bh
FpLeds	equ	200bh

DLEDS	equ	2011h

ABUSS	equ	2014h

RckA	equ	2016h
CRCSUM	equ	2017h

TPERRX	equ	2019h

TicCnt	equ	201Bh			; WORD Interrupt timer count
RegPtr	equ	201Dh

UiVec	equ	201Fh			; User Interrupt Vector

CtlFlg2	equ	2036h

D_CON	equ	2048h

D_Abort	equ	2061h

D_Read	equ	2067h

D_SDP	equ	2085h

D_RAM	equ	20A0h

D_DvCtl	equ	20A2h
D_DylMo	equ	20A3h
D_OeCnt	equ	20B4h

Aio_Uni	equ	2131h

BDA	equ	2150h			; Boot Device Address
BDF	equ	2151h
TimOut	equ	2152h			; TimeOut counter
UsrClk	equ	2154h

Stack	equ	2280h			; Top of Stack
UsrFWA	equ	2280h			; User First Working Address

; Define the memory segments for the two code blocks
	define	XCON8, ORG=0
	define	H17ROM, ORG=1800h

; Set the ORG 0000H segment active
	segment	XCON8

; Program code start point; CPU power-on/reset starts here
Ram8Go	equ	$
        LD      DE,0
        JP      XInit
        JP      Init

; The last byte of the above code is 00H, which is also the
; 1st byte of the Int1 code - which starts at 0008H.  The resulting
; NOP instruction is meaningless.

Int1	equ	$-1

	CALL	SavAll
	LD	D,0
	JP	Clock

; As above, the last byte of the Int1 code is 00H, a NOP, and
; it overlaps the Level 2 interrupt code at Int2, starting at
; 0010H.  The NOP makes it a meaningless overlap.

Int2	equ	$-1
	call	SavAll
	LD	A,(DE)
	JP	StpRtn

; Interrupt handlers for Int3 - Int7.  First,
; verify the correct address counter.

	  if  $ != 0018h
	.error "* Address Error @ 0018H *"
	  endif

Int3:	JP	UiVec + 6		; Jump to user routine
	DB	"44470"			; Heath part # 444-70

Int4:	JP	UiVec + 9		; Jump to user routine
	DB	"@JGL@"			; Mr. Letwin's initials -- "Support Code"

Int5:	JP	UiVec + 12		; Jump to user routine

;       Subroutine      Dly - Delay a bit
;       Inputs:         A = mSec delay count/2
;
; For instance, LD A,10/2 will give a 10mSec delay count.

Dly:	PUSH	AF			; Save count
	XOR	A			; Don't sound the horn
	JP	Hrn0			;  but process in same manner

Int6:	JP	UiVec + 15		; Jump to user routine

Go_:	LD	A,0D0H			; Off monitor mode light
	JP	Sst1			; Return to user program

Int7:	JP	UiVec + 18		; Jump to user routine

;============================================================================
;	Init - Initialize System
;
;  Init is called whenever a hardware master=clear is initiated.
;
;  1 - Setup PAM/8 control cells in RAM,
;  2 - Decode the amount of memory in-system,
;  3 - Set Stack pointer, and
;  4 - Enter the monitor loop.
;
;	Entry from Master Clear
;	Exit to PAM/8 mainLoop
;============================================================================
Init:	LD	A,(DE)			; Copy PRSROM into RAM
        LD      (HL),A			; Move byte
        DEC     HL			; Decrement destination
        INC     E			; Increment source
        JP      NZ,Init			; Loop if not done
        LD      D,4			; Search increment = 400H
        LD      HL,2000h		; HL gets 1st location to test
; Determine memory limit
        JP      XInit1

	DB	"HEATH"			; Corporate ownership

; Return to in-line code.  HL Set to first non-existant RAM location
Init2:  DEC     HL
        LD      SP,HL			; Set Stack pointer to memory limit - 1
        LD      HL,DefPC
        PUSH    HL			; Set *PC* value on stack
        CALL    Patch1			; Tape UART/Auto boot
        PUSH    HL			; Set return address
        XOR     A			; Leave addresses the same & A = 0
; Fall through...

;============================================================================
;       Subroutine      SavAll
;
; SavAll is called when an interrupt is accepted, in order to
; save the contents of the registers on the Stack.
;
;	Entry	Called directly from interrupt handler
;	Exit	All registers pushed on Stack.
;
;  If not yet in Monitor Mode, REGPTR = Address of registers on Stack.
;
;  (DE) = Address of CTLFLG.
;============================================================================
SavAll:	EX	(SP),HL			; Set HL on stack top
	PUSH	DE
	PUSH	BC
	PUSH	AF
	EX	DE,HL			; DE gets return address
	LD	HL,10
	ADD	HL,SP			; HL = address of users SP
	PUSH	HL			; Set on stack as register
	PUSH	DE			; Stack the return address
	LD	DE,CtlFlg
	LD	A,(DE)			; A gets CtlFlg contents
        CPL
        AND     30H			; Save register addr if user or single-step
        RET     Z			; Exit if interrupt of monitor loop
        LD      HL,2
        ADD     HL,SP			; HL = address of 'StackPtr' on stack
        LD      (RegPtr),HL
        RET

;============================================================================
;	CUI - Check for User Interrupt processing
;
;  CUI is called to see if the user has specified processing for the
; clock interrupt.
;============================================================================
CUI1:	LD	A,(BC)			; A gets .MFlag contents
	RRCA
	CALL	C,UiVec			; If specified, transfer to user

; Return to program from Interrupt.
IntXit:	POP	AF			; Remove fake 'Stack Register'
	POP	AF			; Restore register contents
	POP	BC
	POP	DE
	POP	HL
	EI
	RET

;============================================================================
;	Clock - Process Clock Interrupt
;
;  Clock is entered whenever a Millisecond clock interrupt is
; processed.
;
;  TicCnt is incremented every interrupt, forming a 1mSec counter.
;============================================================================
Clock:	LD	HL,(TicCnt)
        INC     HL
        LD      (TicCnt),HL		; Increment TicCnt

;	Refresh front panel
;
;	  This code displays the appropriate pattern on the
;	front panel LEDs.  The LEDs are painted in reverse order,
;	one per interrupt.  First, # 9 is lit, then # 8, etc.

        LD      HL,MFlag
        LD      A,(HL)
        LD      B,A			; B = current flag
        AND     40H			; Front panel refresh wanted?
        INC     HL
        LD      A,(HL)			; A = CtlFlg
        LD      C,D			; C=0 in case no panel display
        JP      NZ,Clk3			; Go if no front panel refresh
        INC     HL			; Point to Refresh Index
        DEC     (HL)			; -1 for digit index
        JP      NZ,Clk2			; Go if not wrapped around
        LD      (HL),9			; Esle, wrap around to start
Clk2:	LD	E,(HL)
        ADD     HL,DE			; HL -> address of pattern
        LD      C,E
Clk3:	OR	C
        OUT     (0F0H),A		; Select digit
        LD      A,(HL)
        OUT     (0F1H),A		; Select segment

; See if time to decode display values
        LD      L,low TicCnt
        LD      A,(HL)
        AND     31			; Every 32 interrupts
        CALL    Z,UFD			; Update front panel displays

; Exit clock interrupt
        LD      BC,CtlFlg
        LD      A,(BC)			; A = CtlFlg
        AND     20H
        JP      NZ,IntXit		; Go if in Monitor Mode
        DEC     BC
        LD      A,(BC)			; A = _MFlag
        RLA
	JP	C,Clk4			; Skip it

; Not in Monitor Mode.  Check for Halt
	LD      A,10
        CALL    LRA_			; Locate P-Reg address
        LD      E,(HL)
        INC     HL
        LD      D,(HL)			; DE = PC contents
        DEC     DE
        LD      A,(DE)
        CP      76H			; Check for Halt
        JP      Z,Error			; If Halt, be in monitor mode

; Check for 'Return to Monitor' key entry
Clk4:	IN	A,(0F0H)
	CP	2EH			; See if '0' and '#'
	JP	NZ,CUI1			; If not, allow user clock processing

;============================================================================
;	Error - Command Error
;
;  Error is called as a 'bail out' routine.  It resets the operational mode
; and restores the stack pointer.
;============================================================================
Error:  LD      HL,MFlag
        LD      A,(HL)			; A = _MFlag
        AND     0BDH			; Re-enable displays
        LD      (HL),A
        INC     HL
        LD      (HL),0F0H		; Restore CtlFlg
        EI
        LD      HL,(RegPtr)
        LD      SP,HL			; Set Stack Pointer to empty state
        CALL    Alarm			; Alarm for 200mSec
; Fall through...

;============================================================================
;	MTR - Monitor Loop
;
; This is the main executive loop for the Front Panel Emulator
;============================================================================
MTR:    EI      
MTR1:   LD      HL,MTR1
        PUSH    HL			; Set MTR1 as the return address
        LD      BC,DspMod
        LD      A,(BC)
        AND     1			; A = 1 if altered
	CPL
	LD	(DspRot),A		; Set flag bit if DspMod altered

; Read Keypad
	CALL	RCK			; Read console keyset
        LD      HL,(ABUSS)
        CP      10
        JP      NC,Mtr4			; Go if in 'always valid' group
        LD      E,A			; Save value
        LD      A,(BC)			; Get DspMod value
        RRCA    
	JP      C,Mtr5			; Go if in Alter mode
	LD	A,E			; Code to A
; Have a command - not a value

Mtr4:	SUB	4			; A = command
	JP	C,ExtCmd	        ; Extended commands
	LD	E,A
	PUSH	HL			; Save ABUSS value
	LD	HL,MtrA			; Point to command table
	LD	D,0			; Put command offset in DE
	ADD	HL,DE			; Add cmd offset to cmd tbl pointer
	LD	E,(HL)			; Get command tble entry
	ADD	HL,DE			; HL gets address of cmd tbl entry
	EX	(SP),HL			; Cmd adress on stack, ABUSS to HL
	LD	DE,RegI			; Point to register index
	LD	A,(BC)			; A = DspMod
	AND	2			; Set 'Z' if memory mode
	LD	A,(BC)			; Reload DspMod to A
	RET				;  and 'exit' to command

MtrA:					; Command Jump table
	db	Go-$			; 4 - Go
	db	In-$			; 5 - Input
	db	Out-$			; 6 - Output
	db	SStep-$			; 7 - Single Step
	db	RMem-$			; 8 - Cassette Load (read)
	db	WMem-$			; 9 - Cassette Dump (write)
	db	Next-$			; + - Next
	db	Last-$			; - - Last
	db	Abort-$			; * - Abort
	db	R$W-$			; / - Display/Alter
	db	MemM-$			; # - Memory Mode
	db	RegM-$			; . - Register Mode

;============================================================================
;	Mtr5	Process Memory/Register Alterations
;
;	This code is entered if:
;
;	- We are in Alter mode, and
;	- A key from 0-7 was entered.
;============================================================================
Mtr5:	RRCA
	LD	A,E			; A = value
	JP	C,Mtr6			; Go if it's a register
        SCF				; Flag 1st digit in A
	CALL	IOB			; Input octal byte
	INC	HL			; Display next location
; Fall through...

;============================================================================
;	SAE	Store ABUSS and Exit
;============================================================================
SAE:	LD	(ABUSS),HL
	RET

;============================================================================
;	H89Pin	H89-compatible PIN routine
;============================================================================
	  if  $ != 0137h
	.error "* Address Error at H89Pin (0137H) *"
	  endif

H89Pin:	JP	PIN

; Continue with MTR code from Mtr5 above
Mtr6:	PUSH    AF		; Save code
	CALL	LRA		; Locate Register Address
	AND	A
	JP	Patch2

	DB	0,0		; Reserve space

	  if  $ != 0144h
	.error "* Address Error at RegM (0144H) *"
	  endif
;============================================================================
;	RegM	Enter Register Display Mode
;
;	On entry, A = (DspMod)
;============================================================================
RegM:	LD      A,2			; Set 'Display Register' mode
        LD      (BC),A
        DEC     BC			; Point to DSProt
        XOR     A
        LD      (BC),A
	CALL   RCK			; Read key entry
        DEC     A			; Displace
        CP      6
        JP      NC,Error		; Go if not 1-6
	RLCA
	LD      (DE),A			; Set new register index
	RET

;============================================================================
;	R$W	Toggle Display/Alter mode
;
;	On entry:	A = DspMod
;			BC points to DspMod
;============================================================================
R$W:	XOR     1			; Set 'other' mode
        LD      (BC),A			;  and save new DspMod
        RET

;============================================================================
;	Next	Increment Display Element
;
;	On entry:	HL points to ABUSS
;			DE has address of RegInd (Register Index)
;============================================================================
Next:	INC     HL
        JP      Z,SAE			; If memory mode, store ABUSS and exit
        LD      A,(DE)			; Get (RegI) value
        ADD     A,2
        LD      (DE),A			; Wrap to SP
        CP      12 
        RET     C			; If not too large, exit
        XOR     A			; Else wrap around
        LD      (DE),A			;  and save new index
; Fall through...

;============================================================================
;	Abort	Exit current process
;============================================================================
Abort:	RET

;============================================================================
;	Last	Decrement Display Element
;
;	On entry:	HL points to ABUSS
;			DE points to RegInd (Register Index)
;============================================================================
Last:	DEC	HL
	JP	Z,SAE			; If memory mode, store and exit
        LD      A,(DE)

; We're in register mode
        SUB	2			; -2 for register #
        LD      (DE),A			; Save in index pointer
        RET	NC			; Exit if no wrap around
	LD	A,10			; Underflow, so set PC as
        LD      (DE),A			;  current register index
	RET

;============================================================================
;	MemM	Enter Display Memroy Mode
;
;	On Entry, BC has address of DspMod (Display Mode).
;============================================================================
MemM:	XOR	A
	LD	(BC),A			; Set DspMod to 'Display Memory'
	DEC	BC			; Point to DsProt
        LD      (BC),A			; Set all periods on
        LD      HL,ABUSS+1
        JP      IOA			; Input Octal address

;============================================================================
;	In	Input Data Byte
;	Out	Output Data Byte
;
;	On entry, HL points to ABUSS (Address Bus)
;============================================================================

In:	LD	B,0DBH			; 'IN' instruction

; The "DB 11H" below is the precessor to LD DE,NNNN.  As a result, DE will be
; loaded with the code generated by the LD B, 0D3H code, which is meaningless.
; The end result is that the LD B,0D3H code is ignored.
	 db	11h			; Skip next instruction

Out:	ld	b,0D3H			; 'OUT' instruction
	LD      A,H			; Put value in A
	LD      H,L			; Port in H
	LD      L,B			; IN/OUT instruction in L
	LD	(IoWrk),HL		; Store 'IN port' at IoWrk
	CALL	IoWrk			;  and execute the code
	LD      L,H			; Put port in L
	LD      H,A			; Value in H
	JP      SAE			; Store ABUSS and Exit

;============================================================================
;	Go	Return to User Mode
;============================================================================
Go:	JP	Go_			; Routine is in waste space

;============================================================================
;	SStep	Single Step Instruction
;============================================================================
SStep:	DI
	LD	A,(CtlFlg)		; Get control Flag
	XOR	10H			; Disable sngl step inhibit
	OUT	(0F0H),A		; Prime signel step interrupt
Sst1:	LD	(CtlFlg),A		; Set new CtlFlg value
	POP	HL			; Clear rtn address from stack
	JP	IntXit			; Rtn to user routine for step

;============================================================================
;	StpRtn	Signle Step Return
;
;	On entry:	DE points to CtlFlg
;============================================================================
StpRtn:	OR	16			; Disable single step interrupt
	OUT	(0F0H),A		; Turn off signle step enable
	LD	(DE),A			; Set new CtlFlg
	AND	20H			; See if in Monitor Mode
	JP	NZ,MTR			;  and go to MTR loop if so
	jp	UiVec+3			; Else, transfer to user's routine

;============================================================================
;	RMem	Load Memory from Tape
;============================================================================
RMem:	LD	HL,TPABT
	LD	(TPERRX),HL		; Setup error exit address
; Fall through...

;============================================================================
;	Load	Load memory from Tape
;
;	  Read the next record from cassette tape, using the load address
;	in the tape record.
;
;	Entry:	HL = Error Exit address
;	Exit	User P-Reg (on stack) set to entry address
;		To caller if OK
;		To Error Exit (HL) is tape errors detected
;============================================================================
Load:	LD	BC,0FE00h		; Required type and #
Load0:	CALL	SRS			; Scan for record start
        LD      L,A			; (HL) = count
        EX      DE,HL			; DE = count, HL = type and #
        DEC     C			; - next #
        ADD     HL,BC
        LD      A,H
        PUSH    BC			; Save type and #
        PUSH    AF			; Save type code
        AND     7FH			; Clear end flag bit
        OR      L
        LD      A,2			; Assume sequence error
        JP      NZ,TpErr		;  and go if wrong type or sequence
	CALL	RNP			; Read address
        LD      B,H
        LD      C,A			; BC = P-Reg address
        LD      A,10
        PUSH    DE
	CALL	LRA_			; Locate Register address
        POP     DE
        LD      (HL),C			; Set P-Reg in mem
        INC     HL
        LD      (HL),B
	CALL	RNP			; Read address
        LD      L,A			; HL = Address, DE = count
        LD      (Start),HL

Load1:	CALL	RNB			; Read next byte
	LD	(HL),A
        LD      (ABUSS),HL		; Set ABUSS for display
        INC     HL
        DEC     DE
        LD      A,D
        OR      E
        JP      NZ,Load1		; Go if more remaining

	CALL	CTC			; Check tape checksum
        POP     AF			; File type byte
        POP     BC			; Last type, Last #
        RLCA    
        JP      C,TFT			; If done, turn off tape
        JP      Load0			; else read next record

;============================================================================
;	Dump	Dump Memory to Mag Tape
;
; Dump specified memory to mag tape.
;
;	Entry	(Start) = Start address
;		(ABUSS) = End Address
;		User PC = Entry Point address
;============================================================================
	  if  $ != 01fch
	.error "* Address Error @ WMem *"
	  endif

WMem:	LD	HL,TPABT
	LD	(TPERRX),HL		; Set up error exit

Dump:	LD	A,1
	OUT	(0F9H),A		; Set up tape control
	LD      A,16H			; Sync character
	LD      H,32			; # of sync characters
WMem1:	CALL	WNB			; Write next byte
	DEC	H
	JP	NZ,WMem1		; Loop for 32 sync chars
	LD      A,2			; STX char
	CALL	WNB			; Write STX
        LD      L,H			; HL = 0
        LD      (CRCSUM),HL
        LD      HL,8101h
	CALL	WNP			; Write header
        LD      HL,(Start)
        EX      DE,HL			; DE gets Start address
        LD      HL,(ABUSS)		; HL has end address
        INC     HL
        LD      A,L
        SUB     E
        LD      L,A
        LD      A,H
        SBC     A,D
        LD      H,A			; HL has byte count
	CALL	WNP			; Write byte count

        PUSH    HL
        LD      A,10
        PUSH    DE
	CALL	LRA_			; Locate P-Reg address
        LD      A,(HL)			
        INC     HL
        LD      H,(HL)
        LD      L,A			; HL = contents of PC
	CALL	WNP			; Write header
        POP     HL			; HL = Address
        POP     DE			; DE = Count
	CALL	WNP

WMem2:	LD	A,(HL)
	CALL	WNB			; Write byte

        LD      (ABUSS),HL		; Set address for display
        INC     HL
        DEC     DE
        LD      A,D
        OR      E
        JP      NZ,WMem2		; Loop if more to go

; Write Checksum
	LD	HL,(CRCSUM)
	CALL	WNP			; Write it
	CALL	WNP			; Flush it
; Fall through...

;============================================================================
;	Subroutine	TFT - Turn Off Tape
;
;  Stop the tape transport.
;============================================================================
TFT:	XOR	A
	OUT	(0F9H),A		; Turn off tape
; Fall through...

;============================================================================
;	Subroutine	Alarm
;
;	Entry:	A = Millisecond count / 2
;============================================================================
Alarm:	LD	A,200/2		; 200 mSec beep
; Fall through...

;============================================================================
;	Subroutine	Horn
;
; Sound horm for mSec length in A.
;============================================================================
Horn:	PUSH	AF			; Save count
	LD	A,80H			; Turn on speaker
Hrn0:	EX	(SP),HL			; Save HL, H = count
	PUSH	DE
	EX	DE,HL			; D = loop count
	LD	HL,CtlFlg
	XOR	(HL)
	LD	E,(HL)			; E = old CtlFlg value
	LD	(HL),A			; Turn on horn
	LD	L,low TicCnt
	LD	A,D			; A = cycle count
	ADD	A,(HL)
Hrn2:	CP	(HL)			; Wait required TicCnt times
	JP	NZ,Hrn2			;  loop if not done
	LD	L,9
	LD	(HL),E			; Turn horn off
	POP	DE
	POP	HL
	RET

;============================================================================
;         Subroutine	CTC - Verify Checksum
;
;	Entry	Tape positioned just before CRC
;	Exit	To caller if OK
;		To TpErr if bad
;============================================================================
CTC:	CALL	RNP			; Read next Pair
        LD      HL,(CRCSUM)
        LD      A,H
        OR      L
        RET     Z			; Exit if OK
        LD      A,1			; Checksum Error code

;============================================================================
;	Subroutine	TpErr - Process Tape Error
;
;  Display error number in low byte of ABUSS
;
; If error # even, don't allow #
; If error # odd, allow #
;============================================================================
TpErr:	LD	(ABUSS),A		; Save the error code
	LD	B,A
	CALL	TFT			; Turn off tape
	 db	0e6h			; MI.ANI - Fall through w/carry clr

Ter3:	LD	A,B
	RRCA    
	RET	C			; Return if OK

; Beep and flash error #

Ter1:	CALL	C,Alarm			; Alarm if proper time
	CALL	TpXit			; See if *
        IN      A,(0F0H)
        CP      2FH			; Check for '#'
        JP      Z,Ter3			;  and go if '#' pressed
        LD      A,(TicCnt+1)
        RRA     			; C set if 1/2 second
        JP      Ter1

;============================================================================
;	Subroutine	TPABT - Abort Tape Load or Dump
;
; Entered when Loading or Dumping, and the '*' key is struck.
;============================================================================
TPABT:	XOR	A
	OUT	(0F9H),A		; Turn off tape
	JP	Error

;============================================================================
;	Subroutine	TpXit - Check for User Forced Exit
;
; TpXit checks for a '*' keypad entry.  If so, take the tape driver
; abnormal exit.
;
;	Exit:	Via RET if not '*'
;		  A = port status
;		To (TpErrx) if '*' pressed
;============================================================================
TpXit:	IN	A,(0F0H)		; Read keypad
        CP	01101111b		; '*' character?
        IN      A,(0F9H)		; Read tape status
        RET     NZ			; Not '*', so return with status
        LD      HL,(TPERRX)		; Get tape error handler address
        JP      (HL)			;  and execute

;============================================================================
;	Subroutine	SRS - Scan Record Start
;
; SRS scans bytes until it recognizes the start of a record.
; This requires at least 10 sync characters and 1 STX character.
; The CRC-16 is then initialized.
;============================================================================
SRS:	LD	D,0
	LD      H,D
	LD      L,D
Srs2:	CALL	RNB			; Read next byte
	INC     D
	CP      16H			; Sync byte?
	JP      Z,Srs2			;  Y - get next byte

	CP	2			; STX byte?
	JP      NZ,SRS			;  N - start over

	LD      A,10
	CP	D			; Enough SYNC characters received?
        JP      NC,SRS			;  go if no

	LD	(CRCSUM),HL		; Y - Clear CRC-16
	CALL	RNP			; Read leader
	LD	D,H
	LD	E,A
; Fall through...

;============================================================================
;	Subroutine	RNP - Read Next Pair
;
; Read next two bytes from input device
;============================================================================
RNP:	CALL	RNB			; Read next byte
	LD	H,A			; Save in H
; Fall through...

;============================================================================
;	Subroutine	RNB - Read Next Byte
;
;  RND reads the next single byte from the input device.  The CRC checksum
; is updated.
;============================================================================
RNB:	LD	 A,34H			; Turn on reader for next byte
	OUT	(0F9H),A
Rnb1:	CALL	TpXit			; Check for '*', read status
	AND	2
	JP	Z,Rnb1			; Loop if not ready
	IN	A,(0F8H)		; Else read data
; Fall through...

;============================================================================
;	Subroutine	CRC - Computer CRC-16
;
; CRC computes a CRC-16 checksum from the polynomial:
;
;  (X + 1) * (X^15 + X + 1)
;
;  Since the checksum generated is a division remainder,
; a checksummed data sequence can be verified by running
; the data through CRC, and running the previously obtained
; checksum through CRC.  The resulting checksum should be 0.
;============================================================================
CRC:	PUSH    BC
        LD      B,08H				; B = bit count
        PUSH    HL
        LD      HL,(CRCSUM)
Crc1:	RLCA
        LD      C,A				; C = bit
        LD      A,L
        ADD     A,A
        LD      L,A
        LD      A,H
        RLA     
        LD      H,A
        RLA     
        XOR     C
        RRCA    
        JP      NC,Crc2				; If not to XOR

        LD      A,H
        XOR     80H
        LD      H,A
        LD      A,L
        XOR     05H
        LD      L,A
Crc2:	LD      A,C
        DEC     B
        JP      NZ,Crc1				; Go if more to do

	LD	(CRCSUM),HL			; Store updated checksum/CRC
	POP	HL				; Restore
	POP	BC
	RET

;============================================================================
;	Subroutine	WNP - Write Next Pair
;
; Writes two bytes to the tape drive.
;============================================================================
WNP:	LD      A,H			
        CALL   WNB				; Write 1st byte
        LD      A,L				; 2nd byte in A
; Fall through...

;============================================================================
;	Subroutine	WNB - Write Next Byte
;
; Writes next byte to cassette tape.
;============================================================================
WNB:	PUSH	AF				; Save byte to send
Wnb1:	CALL	TpXit				; Check for '*', read status
	AND	1
	JP	Z,Wnb1				; Loop if more
	LD	A,11H				; Enable transmitter
	OUT	(0F9H),A			; Turn on tape drive
	POP	AF				; Restore byte
	OUT	(0F8H),A			; Output it
	JP	CRC				; Exit via CRC calc

;============================================================================
;	Subroutine	LRA - Locate Register Address

LRA:	LD	A,(RegI)			; Load register index
LRA_:	LD	E,A
        LD      D,0
        LD      HL,(RegPtr)			; Get address
        ADD     HL,DE				; Add index offsetoffset 
        RET					; Exit w/HL = address

;============================================================================
;	Subroutine	IOA - Input Octal Address
;
;	HL has address of double byte entry
;============================================================================
IOA:	CALL	IOB				; Get 1st byte
	DEC	HL				; Point to 2nd byte storage
; Fall through...

;============================================================================
;	Subroutine	IOB - Input Octal Byte
;
; Read one Octal Byte from the keypad
;
;	Entry:	HL = address to put octal byte
;;===========================================================================
IOB:	LD	D,3				; Digit counter
Iob1:	CALL	NC,RCK				; Read keypad
	CP	8
	JP	NC,Error			; Error if input above 7

	LD      E,A				; Keypad value to E
        LD      A,(HL)				; Get current byte contents
        RLCA					; Rotate left 3
        RLCA
        RLCA
        AND     11111000b			; Mask
        OR      E				; Or in new keypad value
        LD      (HL),A				; Replace old value
        DEC     D				; -1 for digit counter
        JP      NZ,Iob1				; Loop for all 3 digits

        LD      A,30/2				; 30mSec beep to acknowledge entry
        JP      Horn

;============================================================================
;	Subroutine	DOD - Decode Octal Display
;
;	Entry:	HL = Address of LED refresh area
;		B = OR pattern to force on bars or periods
;		A = Octal value
;		HL = HEX digit address
;============================================================================
DOD:	PUSH	DE
	LD	D,DODA/256
	LD	C,3
Dod1:	RLA					; Left 3 places
	RLA
        RLA
	PUSH	AF				; Save for next digit
	AND	7				; Mask
	ADD	A,low DODA			; Add offset into table
	LD	E,A				; D = index
	LD	A,(DE)				; Pattern to A
	XOR	B
	AND	7FH
	XOR	B
	LD	(HL),A				; Set in memory
	INC	HL
	LD	A,B
	RLCA
	LD	B,A
	POP	AF				; A = value
	DEC	C
	JP	NZ,Dod1				; Go if more remaining
	POP	DE
	RET

;============================================================================
;	Subroutine	UFD - Update Front Panel Displays
;
; UFD is called by the clock interrupt processor when it is
; time to update the display contents.  Currently, this is done
; every 32 interrupts, or about 32 times a second.
;============================================================================
UFD:	LD	A,2
	AND     B
	RET     NZ				; Exit if not handling update
	LD      L,low DspRot
	LD      A,(HL)
	RLCA
	LD      (HL),A				; Rotate pattern
	LD      B,A
	INC     HL
	LD      A,(HL)				; A= Display Mode (DspMod)
	AND     2
	LD      HL,(ABUSS)
	JP      Z,Ufd1				; Go if displaying memory

; Displaying registers
	CALL	LRA				; Locate register address
	PUSH	HL
	LD	HL,DSPA
	ADD	HL,DE				; Offset HL to register name pattern
        LD      A,(HL)
        INC     HL
        LD      H,(HL)
        LD      L,A				; HL = Register name pattern
        EX      (SP),HL
        OR      H				; Clear Z flag
        LD      A,(HL)
        INC     HL
        LD      H,(HL)
        LD      L,A				; HL = address of reg pair contents

Ufd1:	PUSH    AF
        EX      DE,HL
        LD      HL,ALeds
        LD      A,D
	CALL	DOD				; Format ABANK high half
	LD	A,E
	CALL	DOD				; Format ABANK low half
        POP     AF
        LD      A,(DE)
        JP      Z,DOD				; If memory mode, decode byte value
; Is register display, set register name
        LD      (HL),0FFH			; Clear digit
        POP     HL
        LD      (DLEDS+1),HL
        RET     

;============================================================================
;	Subroutine	RCK - Read Console Keyset
;
;  Called to read a keystroke from the console.  It performs
; de-bounce and auto-repeat.  A 'bip' is sounded when a value
; is accepted.
;
;	KeyPad	  Code	     Returned
;	 Key	  Value	       in A
;	  0	1111 1110	0
;	  1	1111 1100	1
;	  2	1111 1010	2
;	  3	1111 1000	3
;	  4	1111 0110	4
;	  5	1111 0100	5
;	  6	1111 0010	6
;	  7	1111 0000	7
;
;	  8	1110 1111	8
;	  9	1100 1111	9
;	  +	1010 1111	10
;	  -	1000 1111	11
;	  *	0110 1111	12
;	  /	0100 1111	13
;	  #	0010 1111	14
;	  .	0000 1111	15
;
;	Inputs		None
;	Outputs		A has keypad code 0-15 for key in above table
;============================================================================
	  if  $ != 03b0h
	.error "* Address Error at RCK (02B0H) *"
	  endif

RCK:	PUSH	HL
	PUSH	BC
	LD	C,400/20			; Wait 400 mSec
	LD	HL,RckA				; Storage in RAM
Rck1:	IN	A,(0F0H)			; Input keypad value
	LD	B,A
	LD	A,20/2				; Wait 20 mSec
	CALL	Dly
	LD	A,B
	CP	(HL)
	JP	NZ,Rck2				; Go if different key from last
	DEC	C
	JP	NZ,Rck1				; Wait N cycles

Rck2:	LD	(HL),A				; Update last key received byte
	XOR	0FEH				; Invert all but group 0 flag
	RRCA
	JP	NC,Rck3				; Hit Bank 0
	RRCA
	RRCA
	RRCA
	RRCA
	JP	NC,Rck1				; No hit at all

Rck3:	LD	B,A				; B gets key code
	LD	A,4/2				; Bip sound timer
	CALL	Horn				; Make bip
	LD	A,B				; Restore key code
	AND	0FH
	POP	BC
	POP	HL
	RET

;============================================================================
;	Display Segment Coding
;
;	Byte = 76 543  210
;
;	 1		----
;      6   2	       |    |
;        0		----
;      5   3	       |    |
;        4		----
;	      7		     DP
;============================================================================

DSPA:		; Register Index to 7-Segment Pattern
	dw	1001100010100100b	; SP
	dw	1001110010010000b	; AF
	dw	1000110110000110b	; BC
	dw	1000110011000010b	; DE
	dw	1000111110010010b	; HL
	dw	1100111010011000b	; PC

DODA:		; Octal to 7-Segment pattern
	db	00000001b	; 0
	db	01110011b	; 1
	db	01001000b	; 2
	db	01100000b	; 3
	db	00110010b	; 4
	db	00100100b	; 5
	db	00000100b	; 6
	db	01110001b	; 7
	db	00000000b	; 8
	db	00100000b	; 9

	db	0	; Space waster to put PRSROM at correct address

; I/O Routines copied to and used in RAM.
;
; Must Continue to 3777A for proper copy.  The table must also
; be backwards to the final RAM.
PRSROM:
	db	1	; Refresh Index
	db	0	; CtlFlg
	db	0	; .MFlag
	db	2	; DSPMOD Display Register
	db	0	; DSPROT
	db	0	; RegI - Show SP Register
	db	0c9h	; Return instruction

	  if  $ != 0400h
	.error "* Address Error @ 0400H *"
	  endif

;============================================================================
;	XInit1 - Size Memory
;
;  XInit1 is jumped to during PAM-8's memory sizing.
; This routine differes from the standard PAM-8 function
; in that it is non-destructive to what may be in RAM below
; 040000A, and it will not wrap-around in a 64K RAM system.
;
;	Entry:	Jumped to from old Init1
;		(DE) = Search increment
;		(HL) = First RAM search location
;
;	Exit:	(HL) = First location where no RAM found
;			(or zero if RAM through 64K present.)
;		(E) = 0 as required
;============================================================================
XInit1: LD      A,(HL)		; Get RAM contents at current address
        DEC     (HL)		; Change it
        CP      (HL)		; Compare to old value
        LD      (HL),A		; Restore old value
        JP      Z,Init2		; Go if contents did not change - no RAM here
        ADD     HL,DE		; Add the search increment
        JP      NC,XInit1	;  and loop back for next block, if no wrap-around
        JP      Init2		; Search complete

;============================================================================
;	XInit	- Extended Initialization
;
; Decide if there is RAM at 0.  If there is, then copy
; RAM Front Panel and H17 ROM to appropriate locations.
; Then jump back to inline init.
;
;  Modified to only do one move directly to RAM.
;
;	Entry:	(DE) = RAM8GO
;
;	Exit:	(DE) = PRSROM
;		(HL) = PrsRAM + PRSL - 1
;		RAM at 0000 set up if present
;============================================================================
H17Rom		equ	1800h		; H17 ROM Start Address
H17RomL		equ	2*1024		; H17 ROM length

	  if  $ != 040eh
	.error "* Address Error @ XInit (040EH) *"
	  endif

XInit:	XOR     A
	LD      (CtlFlg2),A				; Initialize the flag
; Copy RAM test routine to RAM
	LD      C,XinAL					; Length byte in C
	LD      DE,XinA					; Source pointer
	LD      HL,XinB					; Destination pointer
XIn1:	LD      A,(DE)					; Read source
        LD      (HL),A					; Store destination
        INC     DE					; Bump pointers
        INC     HL
        DEC     C					; -1 for byte moved
        JP      NZ,XIn1					; Loop if not last byte

; Test for RAM at location 0
	LD	HL,Ram8Go				; Point to location 0
	LD	A,(CtlFlg2)				; Get original contents
	LD	B,A					; Save
	OR	0F2H					; Turn on RAM at 0000
	LD      DE,XInit2				; DE = return address
	JP      XinB					; Execute test routine

XInit2:	JP	Z,XInit5				; Go if no change (no RAM at 0000)

; Copy PAM/8 code to RAM at 0000
        LD      BC,Ram8GoLen				; 2K length (0-7FFH)
        LD      DE,Ram8Go				; Location 0

; This part makes no sense, moving data from (DE) to (DE).
; As far as the XinA routine is concerned, the memory map
; is set to ROM + RAM.  In other words, ORG0 is NOT in
; effect, so there is no way to copy from ROM to RAM,
; unless there's one of those odd 'Read ROM, Write RAM'
; sort of mapping schemes in effect.

XIn3:	LD	A,(DE)
        LD      (DE),A
        INC     DE
        DEC     BC
        LD      A,B
        OR      C
        JP      NZ,XIn3					; Loop until all bytes sent

; Copy H17 ROM to RAM at 1800H
        LD      BC,H17RomL				; 2K length
        LD      HL,H17Rom				;  from 800H
XIn4:	LD      A,(DE)					; Read source
        LD      (HL),A					; Write destination
        INC     DE
        INC     HL
        DEC     BC
        LD      A,B
        OR      C
        JP      NZ,XIn4					; Loop until all bytes sent

; Now, put the RAM-only memory map in effect
        LD      A,(CtlFlg2)				; Get current Control Port 2 Flags
        OR      20h					; ORG0 RAM enable
        LD      (CtlFlg2),A				; Save current bits
        OUT     (0F2H),A				; Set ORG0 in place

XInit5:	ld	de,PRSROM				; Restore normal values
        LD      HL,200Ah				; PrsRAM + length - 1
        JP      Init					; Return to in-line code

; This routine is block moved to RAM @ 4002H for execution
XinA:	OUT     (0F2H),A				; Swap in Page0 RAM
        LD      A,(HL)					; Get value and save it
        DEC     (HL)					; Attept to change
        CP      (HL)					; Set NZ if RAM changed
        LD      A,B					; Get original value for 0F2H port
        OUT     (0F2H),A				; Select ROM map
        EX      DE,HL
        JP      (HL)
XinAL	equ	$-XinA

	  if  $ != 0470h
	.error "* Address Error @ ExtCmd *"
	  endif

;============================================================================
;	ExtCmd - Extended Command Processor
;
; Handler for commands 0-3
;============================================================================
ExtCmd: ADD     A,4
        ADD     A,A			; A = 2 * A
        LD      DE,ExtTbl
        LD      L,A
        LD      H,00H
        ADD     HL,DE			; (HL) = Processor address
        LD      A,(HL)
        INC     HL
        LD      H,(HL)
        LD      L,A			; HL = Processor address
        JP      (HL)			; Enter command processor

ExtTbl: dw      Error			; '0' - illegal
        dw      PriBoo			; '1' - Primary Boot
        dw      SecBoo			; '2' - Secondary Boot
        dw      Error			; '3' - illegal

;============================================================================
;	AutoB - Auto Boot
;
; Performs an auto boot of the primary device.
;============================================================================
AutoB:  ld      hl,MFlag
        LD      A,(HL)
        AND     0BDH
        LD      (HL),A			; Enable display update, and refresh
        INC     HL
        LD      (HL),0F0H
        LD      A,-1
        LD      (DspRot),A		; All periods OFF
        EI      
        LD      HL,(RegPtr)
        LD      SP,HL
        JP      PriBoo			; Boot primary device

;============================================================================
;	SecBoo - Secondary Boot
;	PriBoo - Primary Boot
;
;  PriBoo is called to boot from the primary boot device
; as defined in the configuration port IP.COM.  The
; alternate entry, SecBoo, is called to boot from the
; secondary boot device.  If the CN.PRI switch is one,
; the address 170 is the primary device; otherwise,
; address 174 is the boot device.  From there, the
; configuration switch further determines the device type
; with the appropriate masks.

SecBoo: XOR     A
SecBt1: LD      (Aio_Uni),A			; Zero boot unit
        LD      BC,MsgSec
        IN      A,(0F2H)
        CPL					; Invert Primary flag
        JP      Boot				; Boot Secondary device

PriBoo: XOR     A
PriBt1: LD      (Aio_Uni),A
        LD      BC,MsgPri
        IN      A,(0F2H)
; fall through...

Boot:   LD      SP,Stack			; Init the stack ponter
        AND     10H
        IN      A,(0F2H)
        PUSH    AF
        JP      Z,Boot2				; 174 is the device to boot

; Booting 170q device
        LD      A,78H
        LD      (BDA),A				; Save boot device address
        POP     AF
        AND     0CH				; Mask device type
        JP      Z,BtErr				; Go if no device installed at 170
	RRCA
	RRCA					; Device type converted to index
	JP	Boot3

; Booting 174q device
Boot2:  LD      A,7CH
        LD      (BDA),A				; Save boot device address
        POP     AF
        AND     3				; Mask out device type

Boot3:  PUSH    AF				; Save device index
        PUSH    BC
        LD      A,0D0H
        LD      (CtlFlg),A			; Turn off monitor mode
        LD      A,7
        LD      HL,UiVec

Boot4:	LD	(HL),0C3H			; Stuff interrupt vectors
	INC	HL
	LD	(HL),low EiRet			;  with JP EiRet instruction
	INC	HL
	LD	(HL),high EiRet
        INC     HL
        DEC     A
        JP      NZ,Boot4

        XOR     A
        LD      (TimOut),A			; Zero timeout counter
        LD      HL,ROMClk			; H17 ROM clock routine
        LD      (UsrClk),HL
        LD	HL,ClkInt
	LD	(UiVec+1),HL			; Init Clock Interrupt vector
        LD      BC,BootA
        LD      DE,D_CON
        CALL	SMove				; Init ROM/RAM vectors
	 db	BootAL				; 58H - SMove byte count

        LD      HL,D_RAM
        LD      B,1FH
        CALL    ZeroMem

        LD      A,(MFlag)
        OR      3				; Enable clock interrupt, turn off display update
        LD      (MFlag),A
        POP     BC
        LD      DE,ALeds
        CALL	SMove
	 db	MsgLen				; SMove byte count
	LD	L,9-MsgLen
        LD      A,-1
Boot5:  LD      (DE),A				; Blank some RAM
        INC     DE
        DEC     L
        JP      NZ,Boot5

        POP     AF
        LD      DE,Boot6
        PUSH    DE
        LD      (BDF),A				; Set Boot Device Flags
        ADD     A,A
        LD      L,A
        LD      H,00H
        LD      DE,BootTbl			; Point to table entries
        ADD     HL,DE				; Add offset in HL
        LD      A,(HL)				; Put table entry in HL
        INC     HL
        LD      H,(HL)
        LD      L,A
        JP	(HL)				;  and execute

Boot6:	LD	A,(MFlag)
        AND     0FDH				; Turn on Display Update
        LD      (MFlag),A			; Restore original front panel mode
        LD      HL,(UsrClk)
        LD      (UiVec+1),HL			; Clear timeout vector to just user vector
        JP      C,Error				; Boot routines return here
        JP      UsrFWA

BootTbl:
	dw	BH17
	dw	BH47
	dw	BtErr
	dw	BtErr

MsgPri: db      10011000b,11011110b,11011111b   ; 'Pri'
MsgLen  equ     $-MsgPri
MsgSec: db      10100100b,10001100b,10001101b   ; 'SEC'

;============================================================================
;	BtErr - Boot Error
;
; Handle boot errors.
;============================================================================
BtErr:  LD      A,(MFlag)
        OR      2				; Disable display update
        LD      (MFlag),A
        LD      BC,BErrA
        LD      DE,ALeds
        CALL    SMove				; Set up error msg in LEDs
         db     BtErrAl

        LD      BC,0				; Msg display timeout counter
BErr1:  DEC     BC
        LD      A,B
        OR      C
        JP      Z,Error				; Go if timeout complete
        IN      A,(0F0H)
        CP      6FH				; Check for '*' key
        JP      Z,Error				; Go if '*' pressed
        JP      BErr1				;  else loop for timeout

BErrA:  db      206o, 306o, 306o, 362o  ; 'boot'
        db      377o, 377o
        db      214o, 336o, 336o        ; 'Err'
BtErrAl equ     $-BErrA

;============================================================================
;	ClkInt - Clock Interrupt Processor
;
; Processes clock interrupts by:
;	- Checking for abort
;	- Checking for Timeout
;	- Passing the Interrupts on to the User
;
; This clock routine is only to be used at boot time.
;============================================================================
ClkInt: PUSH    AF
        IN      A,(0F0H)			; Read front panel
        CP      6FH				; '*' key pressed?
        JP      Z,Error				; Yes, so abort the boot
        LD      A,(TicCnt)
        AND     A
        JP      NZ,ClkI3			; Go if not time to bump internal timer

        LD      A,(TimOut)			; Bump Time Out counter
        INC     A
        LD      (TimOut),A
        CP      30
        JP      C,ClkI3				; Go if not the end yet

; Timeout error!
        LD      A,(BDF)				; Get boot device flags
        CP      0
        JP      NZ,ClkI1			; Go if not H17 device

; Was H17 boot - abort it
        XOR     A
        LD      (D_DylMo),A
        LD      A,(D_DvCtl)
        AND     80H				; Mask all but Ram/Write
        LD      (D_DvCtl),A
        OUT     (7FH),A				; Turn off motor
        JP      ClkI2

; Was H47 boot - abort it
ClkI1:  CP      1				; H47 boot?
        JP      NZ,ClkI2			; Should never happen
        CALL    OBD_				; Reset H47
         db     2,0

; Restore user clock vector
ClkI2:	LD	HL,(UsrClk)
	LD	(UiVec+1),HL
	EI
	JP	BtErr				; Boot timeout error handler

ClkI3:  POP     AF
        PUSH    HL
        LD      HL,(UsrClk)
        EX      (SP),HL
        RET					; Enter user's clock routine

	blkb	0613h-$, 0			; Fill unused ROM with 00 bytes

;============================================================================
;	H89 COM/DAT
;
; H89COM and H89DAT are provided as commen entry points between
; MTR-89 and RAM8GO.
;
; They must reside as a specific address: 0613H
;============================================================================

          if  $ != 0613h
        .error "* Address Error @ 0613H *"
          endif

H89Dat:	JP	Dat_
	 db	0               ; Space waster
H89Com:	JP	Com_

;============================================================================
;	BH17 - Boot H17
;
;  BH17 boots the specified unit of an H17 disk.  The unit to boot
; is specified in AIO.UNI.
;
;	Entry:	AIO.UNI loaded with H17 unit to boot
;		All H17 RAM vectors intialized
;============================================================================
BH17:   LD	BC,BH17A
        LD	DE,FpLeds+3+2
        CALL	SMove				; Move H17 msg in to display
	 db	BH17AL

        XOR     A
        OUT     (7FH),A				; Turn off disk
        LD      HL,R_SDP
        LD      (D_SDP+1),HL			; Re-vector Set Device Parameter for SY2:

BH170:  LD      E,10				; Watch for 10 holes
BH171:  CALL    BH174				; Watch inter-hole gap
        AND     1
        JP      Z,BH171				; Loop if gap not complete

BH172:  CALL    BH174				; Watch entire hole
        AND     1
        JP      NZ,BH172			; Loop if hole not complete
        DEC     E				; -1 for hole just watched
        JP      NZ,BH171			; Loop if 10 holes not complete

BH173:	CALL	D_Abort
	LD      DE,Stack
        LD      BC,9*256
        LD      HL,0
	CALL	D_Read
        JP	C,BH170				; Go if error reading sectors
	RET

BH174:	LD	A,(Aio_Uni)
	LD	B,A
	INC	B
	XOR	A
	CALL	BitS				; Select device
	OR	10H				; Turn on motor
	OUT	(7FH),A				; Motor on and drive select
	IN	A,(7FH)				; Read drive status
	RET

BH17A:  db      222o, 377o, 363o, 361o  ; 'H 17'
BH17AL  equ     $-BH17A

;============================================================================
;	BH47 - Boot H47
;
;  Boot the specified unit of an H47 disk system.  The unit to boot
; is specified in AIO_UNI.
;
;	Entry:	AIO_UNI = Unit of H47 to boot
;		All H-17 RAM vectors initialized
;============================================================================
BH47:	LD	BC,BH47A
	LD	DE,FpLeds+3+2
	CALL	SMove				; Move H47 msg to display
	 db	BH47AL

BH471:	CALL	BH472
	RET	NC				; Go if no errors at boot

        LD      A,500/2				; Wait 1/2 second
        CALL    Dly
        JP      BH471				; Errors, so try again

BH472:	CALL	OBD_				; Output to boot device
	 db	2,0				; Data to output
	CALL	WDN				; Wait for done
	RET	C				; Try again

; Wait for device ready
BH473:  CALL    RRdy
        RET     C
        CALL    RRdy				; L = ready bits
        RET     C

        LD      A,(Aio_Uni)
        LD      B,A
        XOR     A
        CALL    BitS
        AND     L
        JP      NZ,BH473

; Boot the device
	CALL	COM
	 db	3				; Load Sector Count command
	RET	C

	CALL	DAT				; Output data
	 db	0
	RET	C

        CALL    DAT				; Output transfer count
         db     2				; 2 Sector transfer
        RET     C

        CALL    WDN
        RET     C				; Try again

        CALL    COM				; Output command
         db     7				; Read Command
        RET     C

        CALL    DAT				; Track #
         db     0				; Track 0
        RET     C

        LD      A,(Aio_Uni)
        RRCA    
        RRCA    
        RRCA    
	OR	1				; Start at sector 1
        CALL    Dat_
        RET     C

	LD	DE,UsrFWA
BH474:  CALL	PIN
	JP	C,WDN				; Premature DONE means end, error set if S.ERR
	LD	(DE),A
	INC	DE
	JP	BH474				; Get another byte

BH47A:  db      222o, 377o, 262o, 361o  ; 'H 47'
BH47AL  equ     $-BH47A

;============================================================================
;       Subroutine      COM - Output a Command Byte to H47
;
;       Inputs          Command Byte on Stack
;       Outputs         C set if error, else NC
;============================================================================
COM:	EX	(SP),HL
	LD	A,(HL)			; Get the command byte in A
	INC	HL
	EX	(SP),HL			; Restore return address
Com_:	PUSH	AF
	CALL	WDN
	JP	C,Com1			; Go if error
	POP	AF
        CALL	OBD			; Output to data port
	 db	1
	and	a			; Clear C flag
	ret				;  and exit, no error

Com1:	INC	SP
	INC	SP
	RET

;============================================================================
;       Subroutine      DAT - Output Data to Boot H47 with DTR handshake
;
;       Inputs          Output data byte is on the Stack
;============================================================================
DAT:    EX      (SP),HL
        LD      A,(HL)				; A has data to output
        INC     HL
        EX      (SP),HL
Dat_:   PUSH    AF
        CALL    WTR
        JP      C,Dat1				; Go if error
        POP     AF
        CALL    OBD				; Output to data port
	 db	1
	and	a
	RET

Dat1:	INC	SP				; Ignore saved AF reg
	INC	SP
	RET

	  if  $ != 0702h
	.error "* Address Error @ EiRet (0702H) *"
	  endif

EiRet:  EI      
        RET     

;============================================================================
;	IBD - Input from Boot Device
;
;	Inputs		BDA has Boot Device base address
;			Stack has input port offset index
;	Outputs		A has data byte
;============================================================================
IBD:    EX      (SP),HL
        PUSH    AF
        PUSH    DE
        LD      D,(HL)				; D = port index
        INC     HL
        LD      A,(BDA)
        ADD     A,D
        EX      DE,HL
        LD      H,A				; H = actual output address
        LD      L,0DBH
        LD      (IoWrk),HL			; Stuff instruction and port
        EX      DE,HL
        POP     DE
        POP     AF
        EX      (SP),HL
        JP      IoWrk				; Do the actual input

;============================================================================
;	OBD - Output to Boot Device
; Input data from the boot port as saved at boot time.
;
;	Inputs		BDA has Boot Device base address
;			Stack has output port offset index
;
;============================================================================
; For output data on stack
OBD_:   EX      (SP),HL
        LD      A,(HL)				; Get output data byte
        INC     HL
        EX      (SP),HL

; For Output data in A
OBD:    EX      (SP),HL
        PUSH    AF
        PUSH    DE
        LD      D,(HL)
        INC     HL
        LD      A,(BDA)
        ADD     A,D
        EX      DE,HL
        LD      H,A				; Port address to H
        LD      L,0D3H				; OUT instruction
        LD      (IoWrk),HL
        EX      DE,HL
        POP     DE
        POP     AF
        EX      (SP),HL
        JP      IoWrk				; Do the actual I/O

;============================================================================
;	PIN - Port In
;
;  Input a byte of data from H47 with DTR handshake.
;
;	Outputs		'C' set if Error, else NC
;			A has data byte
;============================================================================
PIN:	CALL	IBD
	 db	0
	AND	0A0H
	JP	Z,PIN				; Not done, not ready to xfer

	AND	20H
	SCF
	RET	NZ				; Error - done before DTR

	CALL	IBD
	 db	1
	AND	A
	RET

;============================================================================
; 	R_SDP - Setup Device Parameters
;
;  Sets up arguments for the specific unit
;
;		D_DvCtl = Motor ON
;		D_TrkPt = Address of device track number
;
;	Entry:		AIO_UNI = Unit number
;============================================================================
R_SDP:	LD	A,10				; 10 Errors max
	LD	(D_OeCnt),A			; Set max error count
	LD	A,(Aio_Uni)
	PUSH	AF
	CP	2				; Units 0/1 ?
	JP	C,R_SDP_
	LD	A,3				; Set to Unit 3
	JP	R_SDP_

;============================================================================
;	RRdy - Read Ready
;
; Check to see if drive spec'd in AIO.UNI is ready.
;
;	Inputs		AIO.UNI has unit #
;	Outputs		L has ready bits
;============================================================================
RRdy:	CALL	COM				; Output command
	 DB	10H
	CALL	PIN
	RET	C				; Exit if drive error
	LD	L,A				; RDY bits in L
	CALL	WDN
	RET

;============================================================================
;	SMove - Short Move (<256 bytes)
;
;	Inputs		BC = Source
;			DE = Destination
;			Byte Count on Stack
;============================================================================
SMove:	EX	(SP),HL
	LD	A,(HL)				; Get byte count
	INC	HL
	EX	(SP),HL
	LD	L,A				; L has byte count
SMove1:	LD	A,(BC)
	LD	(DE),A
	INC	BC
	INC	DE
	DEC	L
	JP	NZ,SMove1			; Move them all
	RET

;============================================================================
;	WDN - Wait for Done
;
;  WDN waits for the done bit to be set.  A timeout is kept track of
; in order that the command may be re-tried.
;
;	Outputs		C set if error or timeout, else NC
;============================================================================
WDN:	LD	BC,WtrA
Wdn1:	DEC	BC
	LD	A,B
	OR	C
	SCF
	RET	Z				; Error exit if timeout
	CALL	IBD
	 db	0
	AND	20H
	JP	Z,Wdn1				; Go if not done
	CALL	IBD				; Error only valid after DONE is set
	 db	0
	AND	1 				; Error flag set?
	SCF
	RET	NZ				; Exit w/H47 error
	AND	A				; Clear error flag
	RET					; Exit no error

;============================================================================
;	WTR - Wait for Transfer Request
;
;  WTR waits for a transfer request.  It checks for DONE first, and
; if it is found, flags an error.  The code will also timeout waiting
; for S_DTR.
;
;	Exit	C set is error, else NC
;============================================================================
WTR:	LD	BC,WtrA				; Load timeout count
Wtr1:	CALL	IBD				; Check for DONE
	 db	0
	AND	20H
	SCF
	RET	NZ				; Exit w/error if done

	DEC	BC
	LD	A,B
	OR	C
	SCF
	RET	Z				; Exit w/error if timeout

	CALL	IBD
	 db	0
	AND	80H				; DTR?
	JP	Z,Wtr1				; Loop if no DTR yet
	RET					; Else, exit with DTR set

WtrA	equ	32000				; Timeout counter

;============================================================================
;	Patch1
;
;  PATCH1 replaces code which intially only intialized the tape UART with
; code which also checks for Auto-Boot.  Since the return address is already
; on the stack, if Auto-Boot is se4t, INIT exits to AuytoB instead of ERROR.
;
;	Inputs:		HL = INIT exit address
;			Tape UART initialized
;============================================================================
Patch1:	LD	A,4EH
	OUT	(0F9H),A			; Set 8 bit, no parity, 1 stop, X16

; Check for auto-boot
	LD	HL,Error
	IN	A,(0F2H)
	AND	80H
	RET	Z				; Exit if no autoboot
	LD	HL,AutoB
	RET
;============================================================================
;	Patch2
;
; Patch2 moves the MTR6 code out of its original place in the ROM to permit
; providing for H89/H8 command PIN routine.
;============================================================================
Patch2:	JP	Z,Error				; Not allowed to alter stack pointer
	INC	HL
	POP	AF				; Restore value and C flag
	JP	IOA				; Input Ocatal address

;============================================================================
;	BitS - Bit Set
;
; Set the specified bit in the accumualtor
;
;	Inputs		A has original value
;			B has bit to set (7=high bit)
;	Outputs		A = original A with bit(B) set
;============================================================================
BitS:	PUSH	BC
	PUSH	AF
	LD	A,80H
	INC	B
Bits1:	RLCA
	DEC	B
	JP	NZ,Bits1
	LD	C,A
	POP	AF
	OR	C
	POP	BC
	RET

;============================================================================
;	ZeroMem - Zero Memory
;
; Zeros a block of memory
;
;	Inputs		HL points to data block to zero
;			B has byte count
;
; Entry at Zro1 allows for a block set to the character in A
;============================================================================
ZeroMem:
	XOR	A				; Set 00 in A

; Fill memory block with character in A
ZroM1:	LD	(HL),A
	INC	HL
	DEC	B
	JP	NZ,ZroM1				; Go if more
	RET

; Secondary Entry Vectors must be at specific addresses
	blkb	07f4h-$, 0	; Fill unused ROM with 00 bytes

DefPC:	JP	PriBt1		; Jump to Primary Boot
	JP	SecBt1		; Jump to Secondary Boot
	JP	$		; H89 compatibility
	JP	$		; H89 compatibility

; This marks the end of the XCON-8 monitor code.
Ram8GoLen	equ	$-Ram8Go

;============================================================================
;	H17 ROM Code
;
;  The original H17 ROM code is written in 8080 pseudo ops, but it presented
; here in Z80 code.  Only 8080 opcodes are used, in order to maintain
; compatability with all H17 ROM versions.
;
;  To meet the requirements of 1) the XCON/8 ROM, and 2) the ZDS 3.68
; Windows compiler, the two portions of XCON/8 (PAM/8 and H17 ROM) are
; compiled at their 'run' addresses (0000H and 1800H, respectively).  However,
; this may create difficulties for the linker in that the resulting code file
; will contain a large block of 0FFH bytes from 800H-17FFH, followed by the
; H17 ROM code at 1800H.  This is the normal result of using the ORG assembler
; directive to compile the code at the appropriate location.
;
;  The ZDS 3.68 Linker has a 'Copy' option that allows a code block compiled
; for one address to be stored at a different address.  A Linker command has
; been imbedded in the MAKE process to copy the "H17ROM" segment from 1800h-1FFFH
; down to 0800-FFFH for includsion in the output file, ready for EPROM burn.
;============================================================================

; Switch segments for ORG addressing
	segment	H17ROM

; H17 ROM starts here at 1800H
	jp	RomBoot

;============================================================================
;	Memory Diagnostic
;============================================================================
	LD	HL,-64
	ADD	HL,SP
	EX	DE,HL
	LD	HL,I$2040
	HALT

; Zero test area
	LD	(D_203E),HL
Mem1:	LD	(HL),00H
	INC	HL
	CALL	C_DEHL
	JP	NZ,Mem1
	LD	B,00H
Mem2:	LD	HL,(D_203E)
	INC	B
Mem3:	INC	(HL)
	LD	A,(HL)
	CP	B
	JP	Z,Mem4
	HALT
	NOP

Mem4:	INC	HL
	CALL	C_DEHL
	JP	NZ,Mem3
	JP	Mem2

;============================================================================
;	S_Comp - String Compare
;
;	Entry:	C has character count
;		DE = pointer to String1
;		HL = pointer to String2
;
;	Exit:	NZ is error, where:
;			 C = Length remaining
;			DE = pointer to mismatch in String1
;			HL = pointer to mismatch in String2
;		C is match, where:
;			C = 0
;			DE & HL point to Stringx + Original C
;============================================================================
S_Comp:	LD	A,(DE)
	CP	(HL)
	RET	NZ
	INC	DE
	INC	HL
	DEC	C
	JP	NZ,S_Comp
	RET

;============================================================================
;	DadA	Add 0,A to H,L
;
; Perform HL = HL + 0,A
;============================================================================
DadA:	PUSH	DE
	LD	E,A
	LD	D,0
	ADD	HL,DE
	POP	DE
	RET

;============================================================================
;	DadA_	Add 0,A to H,L
;
; See DadA above.
;============================================================================
DadA_:	ADD	A,L
	LD	L,A
	RET	NC
	INC	H
	RET

;============================================================================
;	DU66	Unsigned 16 / 16 divide
;
;	HL = BC / DE
;============================================================================
DU66:	LD      A,D				; Two's complement DE
        CPL     
        LD      D,A
        LD      A,E
        CPL     
        LD      E,A
        INC     DE
        LD      A,D
        OR      E
        JP      Z,DU665				; Go if divide by zero

	XOR	A
; Shift DE left until:
;
;	1) DE > BL
;	2) Overflow
DU661:	LD	H,D
	LD      L,E
        ADD     HL,BC
        JP      NC,DU662			; Go if too large

        INC     A
        LD      H,D
        LD      L,E
        ADD     HL,HL
        EX      DE,HL				; DE = DE * 2
        JP      C,DU661				; Go if no overflow

; DE overflowed - put it back
	EX	DE,HL
	DEC	A				; Remove extra count

; Ready to start subtracting.  A = loop count
DU662:	LD	H,B				; HL = working value
	LD	L,C
	LD	BC,0				; BC = result
DU663:	PUSH	AF				; Save loop counter
        ADD     HL,DE
        JP      C,DU664				; Go if subtract OK

; Too far
        LD      A,L				; Add back in
        SUB     E
        LD      L,A
        LD      A,H
        SBC     A,D
        LD      H,A

; Left shift BC
DU664:	LD	A,C
        RLA     
        LD      C,A
        LD      A,B
        RLA     
        LD      B,A

; Right shift DE
        SCF     
        LD      A,D
        RRA     
        LD      D,A
        LD      A,E
        RRA     
        LD      E,A

        POP     AF				; Restore loop count
        DEC     A
        JP      P,DU663				; Loop back if more to do
;
DU665:	EX      DE,HL				; Put remainder in DE
        LD      H,B				;  and result in HL
        LD      L,C
        RET

;============================================================================
;	HLIHL	Load HL indirect through HL
;
; Load HL with the contents of the memory location pointed to by HL.
;============================================================================
HLIHL:	LD	A,(HL)				; Get the low byte
	INC	HL				; Point to high byte
	LD	H,(HL)				; Put high byte in H
	LD	L,A				; Put low byte in L
	RET

;============================================================================
;	C_DEHL - Compare DE to HL
;============================================================================
C_DEHL:	LD	A,E
	XOR	L
	RET	NZ
	LD	A,D
	XOR	H
	RET

;============================================================================
;	CHL	Complement HL
;
; HL = -HL  (two's complement)
;============================================================================
CHL:	LD      A,H
        CPL     
        LD      H,A
        LD      A,L
        CPL     
        LD      L,A
        INC     HL
        RET

;============================================================================
;	INDL	Indexed Load
;
;  Loads DE with two bytes at (HL) + Displacement.  Acts as an indexed
; full word load.
;
;	Entry:	HL points to table address
;		Stack points to two-byte displacement
;	Exit	To Stack+2
;============================================================================
INDL:	EX	(SP),HL			; HL = old Stack, SP = Tbl address
        LD      E,(HL)			; Get displacement from Stack
        INC     HL
        LD      D,(HL)
        INC     HL			; HL now points to actual return address
        EX      (SP),HL			; SP gets return, HL points to table
        EX      DE,HL			; DE points to table, HL has displacement
        ADD     HL,DE			; Offset HL to Table + displacement
        LD      A,(HL)			; Move word at offset address into HL
        INC     HL
        LD      H,(HL)
        LD      L,A
        EX      DE,HL			; DE gets offset word, HL points to table
        RET

;============================================================================
;	Move	Move Data
;
; Will move a block of bytes to a new memory address.  If the move is to a
; lower address, the bytes are moved from first to last.  If the move is to
; a higher address, the move is from last to first.  This is done so that
; an overlapped move will not 'ripple.'
;
;	Entry:	BC = Count
;		DE = Source
;		HL = Destination
;============================================================================
Move:	LD	A,B
	OR	C
	RET	Z			; Exit if count = 0

        LD      A,L
        SUB     E
        LD      A,H
        SBC     A,D
        JP      C,Mov2			; Go if moving to a lower address

; Moving up to a higher address
	DEC	BC
	ADD	HL,BC			; Point to highest destination address
	PUSH	HL
	EX	DE,HL			; HL has source address
	ADD	HL,BC			; Offset to highest source address
	PUSH	HL

Mov1:	LD	A,(HL)			; Get source byte
	LD      (DE),A			; move to destination
	DEC     DE			; Adjust pointers
	DEC     HL
	DEC     BC			;  and counter
	LD      A,B
	AND     A
; The AND A will be 'P' until BC = -1, meaning the move is complete
	JP	P,Mov1			; Loop if more to move

	POP     DE			; Restore source & destination pointsers
	POP     HL
	INC     DE			; Bump to next address
	INC     HL
	RET				;  and exit

; Moving to a lower address
Mov2:	LD	A,(DE)			; Get source byte
	LD	(HL),A			; Store in destination
	INC     HL			; Bump pointers
	INC     DE
	DEC     BC			; Dec counter
	LD      A,B
	OR      C
	JP      NZ,Mov2			; Loop back if more to move
	RET

;============================================================================
;	MU10	Unsigned 16-bit multiply by 10
;
; (HL) = (DE) * 10
;
; 'C' set if overflow
;============================================================================
MU10:	EX	DE,HL			; Multiplier in HL
	ADD	HL,HL			; HL * 2
	RET	C			; Exit if overflow
	LD	D,H			; Put *2 value in DE
	LD	E,L
	ADD	HL,HL			; HL = HL * 4
	RET	C
	ADD	HL,HL			; HL = HL * 8
	RET	C
	ADD	HL,DE			; HL = HL *10
	RET

;============================================================================
;	MU66	Unsigned 16 x 16 multiply
;
;	Entry:	BC = Multiplicand
;		DE = Multiplier
;
;	Exit with result in HL, 'Z' set if NO overflow.
;============================================================================
MU66:	XOR	A
	PUSH    AF			; Save overflow status
	LD      HL,0			; Clear result accumulator
MU661:	LD      A,B
	RRA     
        LD      B,A
        LD      A,C
        RRA     
        LD      C,A
        JP      NC,MU662		; Go if bit clear
        ADD     HL,DE
        JP      NC,MU662		; Go if no overflow
        POP     AF
        INC     A
        PUSH    AF

MU662:	LD      A,B
        OR      C			; See if multiplier is 0
        JP      Z,MU663

        EX      DE,HL
        ADD     HL,HL
        EX      DE,HL
        JP      NC,MU661

        POP     AF
	INC     A
        PUSH    AF
        JP      MU661
MU663:	POP	AF
        RET

;============================================================================
;	MU86	Multiply 8 x 16 unsigned
;
; Multiply a 16-bit value by an 8-bit value.
;
;	Entry:	A has multiplier
;		DE has multiplicand
;
;	Exit with result in HL.  'Z' set if NO overflow.
;============================================================================
MU86:	LD	HL,0				; Clear result
	PUSH	BC
	LD	B,H				; B = overflow flag
MU860:	OR	A				; Ensure 'C' clear
MU861:	RRA
	JP	NC,MU862			; Go if not to add
	ADD	HL,DE
	JP	NC,MU862			; Go if no overflow
	INC	B
MU862:	OR	A
	JP	Z,MU863				; Go if done
	EX	DE,HL
	ADD	HL,HL
	EX	DE,HL
	JP	NC,MU861			; Loop if no overflow
	INC	B
	JP	MU860

MU863:	OR	B				; Set 'Z' flag if no overflow
	POP	BC				; Restore
	RET					;  and exit

;============================================================================
;	RstAll	Restore All Registers
;
; Restore all registers from the Stack, and return to the previous caller.
;
;	Entry:	(SP) = AF
;		(SP+2) = BC
;		(SP+4) = DE
;		(SP+6) = HL
;		(SP+8) = RET address
;============================================================================
RstAll:	POP	AF
	POP	BC
	POP	DE
	POP	HL
	RET

;============================================================================
;	SaveAll	Save All Registers
;
; Save all registers on the Stack.
;
;
;============================================================================
SaveAll:
	EX	(SP),HL
	PUSH	DE
	PUSH	BC
	PUSH	AF
	JP	(HL)

;============================================================================
;	TJump	Table Jump
;
; Jump to table address on the Stack, indexed by A.
;============================================================================
TJump:	RLCA					; Index * 2
TJump_:	EX	(SP),HL				; HL points to table
	PUSH    AF				; Save index count
	CALL	DadA_				; Point HL to indexed entry
	LD	A,(HL)				; Put table entry in HL
	INC	HL
	LD	H,(HL)
	LD	L,A
	POP	AF				; Restore index
	EX	(SP),HL
	RET

;============================================================================
;	TBra	Branch Relative Through Table
;
; Uses the supplied index (in A) to select a byte from the jump table.  The
; contents of this byte are added to the address of the byte, yielding the
; jump/branch address.
;============================================================================
TBra:	EX	(SP),HL				; Tbl pointer in HL
	PUSH	DE				; Save
	LD	E,A				; Index to DE
	LD	D,0
	ADD	HL,DE				; Point HL to indexed tbl entry 
	LD	E,(HL)				; Get entry offset
	ADD	HL,DE				; Add to current address
	POP	DE				; Restore
	EX	(SP),HL				; (HL) to stack
	RET					;  exit to indexed pointer

;============================================================================
;	TBLS	Table Search
;
;  Search a table with the following format:
;
;		db	Key1, Value1
;		db	Key2, Value2
;		...
;		db	KeyN, ValueN
;		db	0
;============================================================================
TBLS:	PUSH	BC
	LD	B,A				; Save search key
TBL1:	LD	A,(HL)				; Get table entry key
	CP	B
	JP	Z,TBL2				; Go if table key = search key

	AND	A				; Test for end of table
	INC	HL				; Bump to next table entry
	INC	HL
	JP	NZ,TBL1				; Go if not end of table

; End of table, Key not found
	DEC	HL				; Back to last table entry
	DEC	HL
	OR	H
	LD	A,0				; Flag search key not found
TBL2:	POP	BC
	INC	HL				; Point to Valuex for found entry
	RET


;============================================================================
;	TypTxt	Type Text on the system console
;
; Embedded NULL characters indicated a carriage return/linefeed.  A byte with
; bit 7 set is the last byte in the message string.  Text is on the Stack.
;
;============================================================================
TypTxt:	EX	(SP),HL				; HL points to message
	CALL	TypTx_				; Display the message
	EX	(SP),HL				; Restore stack pointer
	RET

TypTx_:	LD	A,(HL)				; Get next character
	AND     7FH				; Clear bit 7

	RST     38H				; SYSCALL
	 db	2				; .SCOUT function
	CP      (HL)				; Same as before?
	INC     HL
	JP      Z,TypTx_			; Go if mpre remaining
	RET					; Else, bit 7 was set, so return

;============================================================================
;	UDD	Unpack Decimal Digits
;
;  Converts a 16-bit value into a specified number of decimal digits.  The
; result is zero-filled.
;
;	Entry:	BC = Address value
;		 A = Digit Count
;		HL = Memory Address for storing
;
;	Exit with HL = HL + A
;============================================================================
UDD:	CALL	DadA
	PUSH	HL				; Save final HL value
UDD1:	PUSH	AF
	PUSH	HL
	LD	DE,10
	CALL	DU66				; HL = Value / 10
	PUSH	HL
	POP	BC				; BC = remainder
	POP	HL
	LD	A,"0"				; ASCII zero
	ADD	A,E				; Add remainder
	DEC	HL
	LD	(HL),A				; Store digit
	POP	AF
	DEC	A
	JP	NZ,UDD1				; Go if more to go
	POP	HL
	RET

;============================================================================
;	Zero	Zero a block of memory
;
; Zero a block of memory, starting at (HL) for (B) bytes.
;============================================================================
Zero:	XOR	A
Zro1:	LD	(HL),A
	INC	HL
	DEC	B
	JP	NZ,Zro1				; Go if more
        RET

;============================================================================
;	WDR	Write Disable RAM
;
;  Called to disable the write ability of the H17 Floppy RAM.
;============================================================================
WDR:	PUSH	AF
	DI
	LD	A,(D_DvCtl)
	AND	7FH
WDR1:	LD	(D_DvCtl),A
	OUT	(7FH),A
	EI
	POP	AF
	RET

;============================================================================
;	WER	Write Enable RAM
;
;  Called to enable the write ability of the H17 Floppy RAM
;============================================================================
WER:	PUSH	AF
	DI
	LD	A,(D_DvCtl)
	OR	80H
	JP	WDR1

;============================================================================
;	D_Disk	Device Driver Read Code
;
;	Entry:	BC = Count (in sectors)
;		DE = Address
;		HL = Sector
;
;	Exit to Caller if no errors - 'C' flag clear
;	If errors, 'C' set, exit to S.FASER (Fatal System Error) if
;	Unit 0, else to caller if other unit.
;============================================================================
DWrite:
	LD      A,1				; Set write code
	 db	0FEH				; CP x - Skip next byte
DRead:	XOR	A				; Set Read code
	CALL	SYDD				; Call device driver
	RET	NC				; Exit if no error
	PUSH	AF				; Else, save error code
	LD	A,(Aio_Uni)			; Get unit #
	AND	A				; Test for Unit 0:
	CALL	Z,S_FASER			;  and call if 0:
	POP	AF				; Restore error code
	RET


?_09BD: LD      A,(Aio_Uni)
        PUSH    AF
        XOR     A
        LD      (Aio_Uni),A
         CALL   SYDD
;
         CALL   C,C_210B
;
        POP     AF
        LD      (Aio_Uni),A
        RET     


?_09D0: DB    0,0,0,0,0,0,0,0
        LD      A,(Aio_Uni)
        PUSH    AF

        XOR     A
        LD      (Aio_Uni),A
        INC     A
        JP      J$19C5


?_09E4: LD      A,09H   ; 9 
        SCF     
        RET     


?_09E8: LD      A,0AH   ; 10 
        SCF     
        RET     


?_09EC: LD      A,(HL)
        LD      (HL),00H
        LD      D,L
        LD      L,E
        LD      (HL),A
        LD      A,(D_2129)
        LD      L,A
        LD      (HL),D
        LD      HL,D_2129
        LD      (HL),D
        INC     HL
        LD      (HL),00H
        RET     


?_09FF:  CALL   C_1A85
;
        LD      HL,(D_2127)
        LD      (D_214C),HL
         CALL   C$1B9B
;
        LD      (D$214B),A
        LD      (D_2149),A
        RET     Z
;
        LD      A,(D_2128)
        LD      H,A
        LD      A,(D_2126)
        CP      H
        JP      Z,J$19FF
;
        LD      A,B
        AND     A
        RET     Z
;
        LD      HL,(D_2129)
        LD      A,(D_2127)
        CP      L
        JP      Z,J$1A2D
;
        LD      HL,(D$2125)
        PUSH    AF
        LD      A,(D_2128)
        SUB     H
        CPL     
        INC     A
        CP      B
        JP      C,J$1A39
;
        LD      A,B

        LD      C,A
        LD      HL,D_2128
        ADD     A,(HL)
        LD      (HL),A
        LD      A,C
        LD      HL,D_2149
        ADD     A,(HL)
        LD      (HL),A
        LD      A,B
        SUB     C
        LD      B,A
        POP     AF
        RET     Z
;
        LD      A,B
        AND     A
        RET     Z
;
        LD      A,(D_2127)
        INC     A
        PUSH    AF
         CALL   C_1A85
;
        POP     AF
        CP      L
        JP      Z,J$1A1D
;
        RET     


?_0A5B: LD      HL,(D_2124)
        LD      A,(HL)
        AND     A
        SCF     
        RET     Z
;
        LD      E,L
        INC     L
        LD      (HL),A
        LD      L,A
        CP      D
        RET     Z
;
        JP      NC,J$1A71
;
        LD      E,L
        LD      A,(HL)
        AND     A
        JP      NZ,J$1A65
;
        LD      A,C
        AND     A
        RET     NZ
;
        LD      L,A
        LD      E,L
        LD      L,(HL)
        LD      A,(D$213F)
        AND     L
        RET     Z
;
        LD      A,(HL)
        AND     A
        JP      NZ,J$1A75
;
        LD      L,A
        LD      E,L
        LD      L,(HL)
        RET     


?_0A85: LD      HL,(D_2124)
        LD      A,(D_2127)
        LD      L,A
        LD      L,(HL)
        LD      H,00H
        LD      (D_2127),HL
        RET     


?_0A93:  CALL   C_192C
;
        LD      HL,I_20F9
        LD      A,(HL)
        AND     0FEH
        LD      (HL),A
        LD      HL,(D$20F2)
        LD      B,H

        LD      C,L
        LD      HL,(D_20F0)
        PUSH    HL
        EX      DE,HL
        LD      HL,I$16FF
         CALL   C_1AF2
;
        LD      A,(HL)
        LD      (DE),A
        INC     DE
        DEC     BC
        LD      A,B
        OR      C
        JP      NZ,J$1AAA
;
        POP     BC
        DEC     B
        DEC     B
         CALL   C_1AF2
;
        LD      E,(HL)
         CALL   C_1AF2
;
        LD      D,(HL)
        LD      A,D
        OR      E
        JP      Z,J$1AD3
;
        EX      DE,HL
        ADD     HL,BC
        LD      A,(HL)
        ADD     A,C
        LD      (HL),A
        INC     HL
        LD      A,(HL)
        ADC     A,B
        LD      (HL),A
        EX      DE,HL
        JP      J$1AB9


?_0AD3: LD      HL,(D_20F0)
        EX      DE,HL
        LD      HL,(D$20F6)
        LD      A,(HL)
        OR      01H     ; 1 
        LD      (HL),A
        INC     HL
        INC     HL
        LD      (HL),E
        INC     HL
        LD      (HL),D
        EX      DE,HL
        XOR     A
        LD      (D$20F1),A
        LD      A,(D$20F8)
         CALL   C$1AF1
;
        JP      RstAll


?_0AF1: JP      (HL)


?_0AF2: INC     L
        RET     NZ
;
        PUSH    BC
        PUSH    DE
        PUSH    HL
        EX      DE,HL
        LD      BC,100h
        LD      HL,(D_20F4)
        INC     HL
        LD      (D_20F4),HL
        DEC     HL
         CALL   C_19BD
;
        POP     HL
        POP     DE
        POP     BC
        RET     


?_0B0A: PUSH    DE
        PUSH    BC
        LD      HL,(D_20FE)
	CALL	CHL
;
        EX      DE,HL
        LD      HL,(D$20D0)
        ADD     HL,DE
        LD      (D_20FA),HL
        EX      DE,HL
        LD      HL,(D$20D2)
        LD      A,L
        SUB     E
        LD      L,A
        LD      A,H
        SBC     A,D
        LD      H,A
        JP      C,J$1B3B
;
        PUSH    DE
        LD      (D_20FC),HL
        LD      B,H
        LD      C,L
        LD      HL,(D_2102)
         CALL   C$19D8

        LD      HL,I_20F9
        LD      A,80H
        OR      (HL)
        LD      (HL),A
        POP     DE
        LD      HL,(D_20FE)
        LD      B,H
        LD      C,L
        LD      HL,(D$2104)
         CALL   C_19BD
;
        LD      HL,I_20F9
        LD      A,(HL)
        OR      01H     ; 1 
        LD      (HL),A
        LD      HL,(D_20FA)
        LD      DE,6
        LD      B,H
        LD      C,L
        ADD     HL,DE
        LD      (X_2100),HL
        DEC     HL
        LD      A,(HL)
        DEC     HL
        LD      L,(HL)
        LD      H,A
        ADD     HL,BC
         CALL   C$1B7D
;
        POP     BC
        POP     DE
        RET     


?_0B65: LD      HL,(D_214C)
        LD      A,(D_2126)
        LD      C,A
        LD      A,L
        LD      L,H
        LD      H,00H
        LD      B,H
        ADD     HL,BC
        DEC     A
        JP      NZ,J$1B71
;
        LD      A,(D_2149)
        LD      C,B
        LD      B,A
        XOR     A
        RET     


?_0B7D: LD      D,B
        LD      E,C
        PUSH    DE
        LD      E,(HL)
        INC     HL
        LD      D,(HL)
        INC     HL
        LD      A,D
        OR      E
        JP      NZ,J$1B8B
;
        POP     DE
        RET     

?_0B8B: EX      DE,HL
        ADD     HL,BC
        EX      DE,HL
        EX      (SP),HL
        LD      A,(DE)
        ADD     A,L
        LD      (DE),A
        INC     DE
        LD      A,(DE)
        ADC     A,H
        LD      (DE),A
        EX      DE,HL
        POP     HL
        JP      J$1B7F


?_0B9B: LD      HL,(D_2129)
        LD      A,(D_2127)
        CP      L
        LD      A,00H
        RET     NZ

;
        LD      A,(D_2128)
        CP      H
        LD      A,00H
        RET     NZ
;
        LD      A,03H   ; 3 
        RET     


?_0BAF:  CALL   C_192C
;
        LD      HL,RstAll
        PUSH    HL
        LD      HL,I_20F9
        LD      A,(HL)
        AND     A
        RET     P
;
        AND     7EH     ; "~"
        LD      (HL),A
        LD      HL,(D_20FC)
        LD      B,H
        LD      C,L
        LD      HL,(D_20FA)
        EX      DE,HL
        LD      HL,(D_2102)
        JP      C_19BD

;============================================================================
;	R17_SYDD - System Disk Device Driver
;
;  SYDD is the HDOS System H17 Device Driver
;============================================================================
R17_SYDD:
	AND     A
        JP      Z,D_Read
;
        DEC     A
        JP      Z,J$206D
;
        DEC     A
        JP      Z,J$206A
;
        CP      05H     ; 5 
        JP      C,J_205E
;
        JP      Z,D_Abort
;
        JP      J$205B

;============================================================================
;	R17_Mount - Mount new device
;
; Processes device dependent mounting of a new media.  The volumn serial
; number is read into the volumn table and the heads are homed.
;============================================================================
R17_Mount:
	LD      B,L
	LD      HL,0
	CALL   C_2085
	CALL   C_208B
	LD      HL,(D_20A7)
	LD      (HL),B
	JP      J_205E

;============================================================================
;	R17_Abort - Abort and active I/O
;
;  Causes any on-going I/O to be aborted.
;============================================================================
R17_Abort:
	CALL   C_2085
	CALL   C_208B

;============================================================================
;	R17_XOK - Exit with All OK flag
;============================================================================
R17_XOK:
	XOR     A
R17_XIT:
	PUSH    AF
	LD      A,(D_20A4)
	AND     A
	JP      NZ,J$1BFE
	DI      
	LD      A,(D_DvCtl)
	AND     90H
	OUT     (7FH),A
	LD      (D_DvCtl),A
	LD      HL,(D_CON)
	LD      (D_DylMo),HL
	POP     AF
	EI      
	RET
;============================================================================
;	ROMClk - Process H17 Clock Interrupts
;============================================================================
ROMClk:
	LD      A,(TicCnt)
	RRCA    
	RET     C
	AND     A
	LD      HL,D_DylMo
        JP      NZ,J_1C38
        DEC     A
	ADD     A,(HL)
        JP      NC,J_1C38
        LD      (HL),A
        JP      NZ,J_1C38
        LD      A,(D_DvCtl)
        AND     80H
        LD      (D_DvCtl),A
        OUT     (7FH),A
        INC     HL
        LD      A,(HL)
        SUB     01H     ; 1 
        RET     C
        LD      (HL),A
        RET     

;============================================================================
;	R17_Read - Read from Disk
;
;============================================================================
R17_Read:
	PUSH    HL
	CALL   C_2085
;
        LD      HL,(D_20BB)
        INC     HL
        LD      (D_20BB),HL
        POP     HL
        PUSH    DE
        LD      A,C
        AND     A
        JP      Z,J$1C52
;
        INC     B
        PUSH    BC
         CALL   C_2073
;
        LD      A,01H   ; 1 
         CALL   C_208E
I$0C5A  EQU     $-1
;
         CALL   C_207F
I$0C5C  EQU     $-2
;
        JP      C,J_1CC0
;
        POP     BC
        POP     HL
        LD      A,B
        OR      C
        JP      Z,J_1CCD
;
        PUSH    HL
        PUSH    BC
        DEC     B
        JP      Z,J$1C70
;
        LD      C,00H
        LD      B,C
         CALL   C_2091
;
        JP      C,J$1CB1
;
         CALL   C_2082
;
        LD      (HL),A
        INC     HL
        DEC     B
        JP      NZ,J$1C77
;
        LD      A,C
        AND     A
        JP      Z,J$1C8C
;
         CALL   C_2082
;
        INC     C
        JP      NZ,J$1C85
;
        LD      B,D
         CALL   C_2082
;
        CP      B
        JP      NZ,J$1CBA
;
        POP     BC
        DEC     B
        JP      Z,J_1CCD
;
        EX      (SP),HL
        PUSH    BC
        LD      HL,D_20A1
        INC     (HL)
        LD      A,0AH   ; 10 
        SUB     (HL)
        LD      A,00H
        JP      NZ,J$1C58
;
        LD      (HL),A
        DEC     HL
        INC     (HL)
        EI      
         CALL   C_2076
;
        JP      J_1C56


?_0CB1: LD      HL,I$20B5
         CALL   C_209A
;
        JP      J_1CC0


?_0CBA: LD      HL,I$20B7
         CALL   C_209A
;
         CALL   C_2070
;
        JP      NC,J_1C56
;
        POP     BC
        POP     DE
        LD      A,12H   ; 18 
        JP      J_2064


?_0CCD: POP     HL
       JP      J_205E

R17_ReadR:
	PUSH    HL
        CALL   C_2085
;
        LD      HL,I$19D0
        LD      (D_20A7),HL
        JP      J$1C4A

R17_Write:
	PUSH    HL
         CALL   C_2085
;
        LD      HL,(D_20BD)
        INC     HL
        LD      (D_20BD),HL
        IN      A,(7FH)
        AND     04H     ; 4 
        SCF     
        LD      A,15H
        JP      NZ,J$1D5A
;
        LD      HL,0ffh
        ADD     HL,BC
        LD      B,H
        POP     HL
        PUSH    DE
         CALL   C_2073
;
        LD      A,01H   ; 1 
         CALL   C_208E
;
        PUSH    BC
         CALL   C_207F
;
        POP     BC
        JP      C,J$1D52
;
        POP     HL
        LD      A,(D$204A)
        DEC     A
        JP      NZ,J$1D0E
;
        LD      A,(D$204B)
        LD      C,A
        LD      A,(D$204C)
         CALL   C$2094
;
        LD      A,(HL)
         CALL   C_2097
;
        INC     HL
        DEC     C
        JP      NZ,J$1D1C
;
        LD      A,D
         CALL   C_2097
;
         CALL   C_2097
;
         CALL   C_2097
;
         CALL   C_2097
;
        LD      A,(D_DvCtl)
        OUT     (7FH),A
        DEC     B
        JP      Z,J_205E
;
        PUSH    HL
        LD      HL,D_20A1
        INC     (HL)
        LD      A,0AH   ; 10 
        SUB     (HL)
        LD      A,00H
        JP      NZ,J$1CFF
;
        LD      (HL),A
        DEC     HL
        INC     (HL)
        EI      
         CALL   C_2076
;
        JP      J_1CFD


?_0D52:  CALL   C_2070
;
        JP      NC,J_1CFD
;
        LD      A,13H   ; 19 
        POP     HL
        JP      J_2064

R17_CDE:
	EI      
        CALL   C_208B
;
         CALL   C_2076
;
        AND     A
        LD      HL,(D_20B2)
        INC     HL
        LD      (D_20B2),HL
        LD      HL,D_OeCnt
        DEC     (HL)
        RET     P
;
        DEC     HL
        LD      A,0F6H
        ADD     A,(HL)
        LD      (HL),A
        INC     (HL)
        SCF     
        RET     

R17_DTS:
	PUSH    BC
        LD      BC,0FFF6h
        LD      A,B
        INC     A
        ADD     HL,BC
        JP      C,J$1D7F
;
        LD      (D_RAM),A
        LD      A,L
        ADD     A,0AH   ; 10 
        LD      (D_20A1),A
        POP     BC
        JP      R17_SDT				; J_1D95


?_0D91: INC     (HL)
         CALL   C$2079
;

R17_SDT:
	LD      HL,(D_20A5)
        LD      A,(D_RAM)

        CP      (HL)
        JP      Z,C_2088
;
        JP      P,J$1D91
;
        DEC     (HL)
         CALL   C_207C
;
        JP      R17_SDT				; J_1D95

R17_MAI:
	LD      A,20H   ; " "
;        CP      0AFH
	 db	0feh

R17_MAO:
	XOR	A
	PUSH    HL
        LD      H,A
        LD      A,(D_DvCtl)
        AND     9FH
        OR      H
        OUT     (7FH),A
        POP     HL
        OR      40H     ; "@"
        OUT     (7FH),A
        XOR     40H     ; "@"
        OUT     (7FH),A
        LD      A,(D$204D)

R17_Dly:
	PUSH    HL
        LD      HL,TicCnt
        ADD     A,(HL)
        CP      (HL)
        JP      NZ,J$1DC8
;
        POP     HL
        RET     


LPS0:	CALL   C_2088

R17_LPS:
	LD      A,(D$204E)
        LD      B,A
        LD      A,(D_20A4)
        AND     A
        JP      NZ,J$1DCE
;
        DI      
         CALL   C_2091
;
        JP      C,J$1E15
;
        LD      HL,(D_20A7)
         CALL   C_2082
;
        CP      (HL)
        JP      NZ,J$1E1A
;
        LD      HL,D_RAM
         CALL   C_2082
;
        CP      (HL)
        JP      NZ,J$1E1F
;
        INC     HL
         CALL   C_2082
;
        CP      (HL)
        JP      NZ,J$1E0C
;
        LD      H,D
         CALL   C_2082
;
        CP      H
        RET     Z
;
        LD      L,0B8H
        LD      H,20H   ; " "
         CALL   C_209A
;
         CALL   C_2088
;
        DEC     B
        JP      NZ,J$1DDC
;
        SCF     
        RET


?_0E15: LD      L,0B6H
        JP      J_1E07
?_0E1A: LD      L,0B9H
        JP      J_1E07


?_0E1F: LD      L,0BAH
        JP      J_1E07

R17_RDB:
	IN      A,(7DH)
        RRA     
        JP      NC,R17_RDB
;
        IN      A,(7CH)
        LD      E,A
        XOR     D
        RLCA    
        LD      D,A
        LD      A,E
	RET


R17_SDP:
	LD      A,0AH   ; 10 
        LD      (D_OeCnt),A
        LD      A,(Aio_Uni)
        PUSH    AF
        INC     A
        ADD     A,A
        DI      
        LD      HL,D_DvCtl
        XOR     (HL)
        AND     7FH
        XOR     (HL)
        OR      10H     ; 16 
        LD      (HL),A
        OUT     (7FH),A
        LD      HL,D_20A4
        LD      A,(HL)
        AND     A
        LD      (HL),00H
        JP      NZ,J$1E58
;
        LD      A,(D$204F)
        LD      (HL),A
        DEC     HL
        LD      A,(HL)
        LD      (HL),78H        ; "x"
        AND     A
        INC     HL
        JP      NZ,J$1E65
;
        LD      A,(D$2050)
        LD      (HL),A
        EI      
        POP     AF
        ADD     A,A
        LD      HL,I$20A9
        ADD     A,L
        LD      L,A
        LD      (D_20A5),HL
        INC     HL
        LD      (D_20A7),HL
        RET     

R17_STS:
	EI   
        PUSH    BC
        IN      A,(7FH)
        RRA     
        JP      C,J_1E92
;
        LD      HL,TicCnt
        LD      B,(HL)
        IN      A,(7FH)
        RRA     
        JP      C,J_1E92
;
        LD      A,(D$2051)
        ADD     A,B
        CP      (HL)
        JP      NZ,J$1E81
;
        JP      J$1E9B


?_0E92:  CALL   WNH
;
        LD      A,(D$2052)

         CALL   C$209D
;
        POP     BC
        DI      
        IN      A,(7FH)
        RRA     
        JP      NC,J$1E9D
;
        LD      A,(D$2053)
        JP      C_208E

;============================================================================
;	R17_STZ -  Seek Track Zero
;
;============================================================================
STZ0:	CALL   C_207C

R17_STZ:
	IN      A,(7FH)
        AND     02H     ; 2 
        JP      Z,STZ0
;
        LD      HL,(D_20A5)
        LD      (HL),00H
        RET

;============================================================================
;	WNH - Wait for No Hole
;
;============================================================================
WNH:	IN	A,(7FH)
	RRA
	JP	C,WNH
	LD	A,(D$2054)
        
R17_UDly:
	DEC     A
        JP      NZ,R17_UDly
        RET     

; Wait for Sync Character
R17_WSC:
	LD      A,0FDH
        OUT     (7EH),A
        IN      A,(7EH)
        LD      A,(D_2055)
        LD      D,A
        IN      A,(7FH)
        AND     08H     ; 8 
        JP      NZ,J$1EDE
;
        DEC     D
        JP      NZ,J$1ED1
;
        SCF     
        RET     


?_0EDE: IN      A,(7CH)
        LD      D,00H
        RET     

; Write Sync Pattern
R17_WSP:
	DEC     A
        JP      NZ,R17_WSP

        LD      A,(D_DvCtl)
        INC     A
        OUT     (7FH),A
        XOR     A
         CALL   C_2097
;
        DEC     C
        JP      NZ,C_1EED


        LD      A,0FDH
        LD      D,A
        JP      C_2097

R17_WNB:
	LD      E,A
        IN      A,(7DH)
        AND     A
        JP      P,J$1EFC
;
        LD      A,E
        OUT     (7CH),A
        XOR     D
        RLCA    
        LD      D,A
        RET     


?_0F09: LD      B,A
        DEC     HL
        LD      D,E

RomBoot:
        DI      
        LD      SP,Stack
        LD      BC,58h
        LD      DE,BootA
        LD      HL,D_CON
;         CALL   C$18AA
	call	Move
;
        LD      HL,D_RAM
        LD      B,1FH
         CALL   Zero
;
        LD      (Aio_Uni),A
        OUT     (7FH),A
        INC     A
        LD      (MFlag),A
        LD      HL,UiVec
        LD      (HL),0C3H
        INC     HL
        LD      (HL),17H
        INC     HL
        LD      (HL),1CH
        INC     HL
        ADD     A,A
        JP      P,J$1F30
;
        LD      HL,ROMClk
        LD      (D_2020),HL
        EI      
	CALL	R17_Abort
;
        LD      DE,UsrFWA
        LD      BC,9*256
        LD      HL,0
         CALL   C_1C3F
;
        JP      NC,UsrFWA
;
        HALT

	JP      RomBoot

	  if  $ != 1f5ah
	.error "* Address Error @ BootA (1F5AH) *"
	  endif

BootA	EQU	$
	DW	2*256+120		; Head unsettle and motor on times    
	DB	20			; Guardband count for write    
	DB	10			; Number of zero characters after hold edge    
	DB	128/8			; Two character delay before writing    
	DB	15			; Track-to-track step times    
	DB	20			; Number of trys for correct sector    
	DB	70/4			; 70 mS wait for heat settle    
	DB	1000/4			; 1 second wait for motor on    
	DB	8/2+1			; mS/2 to wait for index hole    
	DB	12/2+1			; mS/2 to wait past index hole    
	DB	20			; UDLY count for hole debounce    
	DB	20			; UDlY count for hole debounce    
	DB	64*25/20		; Loop count for 25 characters    

R17_ErrT:
        INC     (HL)
        RET     

; Jump Vectors.

	JP	R17_SYDD			; HDOS System H17 Device Driver
	JP	R17_Mount			; J$1BE5
	JP	R17_XOK				; J$1BFC
	JP	R17_Abort			; C_1BF6
	JP      R17_XIT			; J$1BFD
	JP      R17_Read			; C_1C3F
	JP	R17_ReadR			; J$1CD1
	JP      R17_Write			; J$1CDE
	JP      R17_CDE			; J$1D5E
	JP      R17_DTS			; J$1D7A
	JP      R17_SDT			; J_1D95
	JP      R17_MAI			; J$1DA9
	JP      R17_MAO			; J$1DAC
	JP      R17_LPS			; J$1DD1
	JP      R17_RDB			; C_1E24
	JP	R17_SDP			; J$1E32
	JP	R17_STS			; J$1E75
	JP	R17_STZ			; J$1EAC
	JP	R17_UDly		; J_1EC2
	JP	R17_WSC			; C_1EC7
	JP	R17_WSP			; J_1EE3
	JP	R17_WNB			; C_1EFB
	JP	R17_ErrT		; J$1F68
	JP	R17_Dly			; C_1DC3

BootAL	equ	$-BootA

;============================================================================
;	DDiag - Drive Diagnostics
;============================================================================
DDiag:	LD      A,19H
        OUT     (7FH),A
        LD      C,0FAH
	LD      A,C
	CALL   R17_Dly
	LD      A,C
	CALL   R17_Dly
        DI      
        CALL   C$1FC6
        EI      
        HALT

DDiag0:	CALL	C_1EED
	LD	BC,3164
DDiag1:	LD	A,"G"
	CALL	R17_WNB
	DEC	BC
	LD	A,B
	OR	C
	JP	NZ,DDiag1

	LD	A,18H
	OUT	(7FH),A
	LD	A,219
	LD	(D_2055),A		; Wait for 68 characters max
	CALL	R17_WSC
	RET	C

        LD	BC,3164-2
DDiag2:	CALL	R17_RDB
	CP	"G"
	RET	NZ
	DEC	BC
	LD	A,B
	OR	C
	JP	NZ,DDiag2
        EI
        HALT

	DB	0,"JGL",0,"HEATH",0

	end
;
; End of File "XCON8.ASM"
;----------------------------------------------------------------------------

